/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.layout;

import java.awt.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class ActorControlPointHandlerImpl extends ActorDelegateImpl
         implements ActorControlPointHandler {
    protected Point _scratchPoint;
    protected int _minimumSize;
    // of actor
    protected int _precision;

    public ActorControlPointHandlerImpl(int precision, int minimumSize, Actor actor) {
        super(actor);
        _precision = precision;
        _minimumSize = minimumSize;
        _scratchPoint = new Point();
    }

    public abstract Object copy();

    protected int fuzzyCompare(int arg1, int arg2) {
        int comparisonValue = arg1 - arg2;
        if (comparisonValue < 0) {
            if (comparisonValue < -_precision) {
                return -1;
            }
            return 0;
        }
        if (comparisonValue > 0) {
            if (comparisonValue > _precision) {
                return 1;
            }
            return 0;
        }
        return 0;
    }

    protected boolean fuzzyEqual(int arg1, int arg2) {
        int comparisonValue = arg1 - arg2;
        if (comparisonValue < 0) {
            if (comparisonValue < -_precision) {
                return false;
            }
        } else {
            if (comparisonValue > 0) {
                if (comparisonValue > _precision) {
                    return false;
                }
            }
        }
        return true;
    }

    public abstract void getControlPoint(int controlPoint, Point returnValue);

    public abstract int getControlPointForPoint(Point inputLocation);

    public int getMinimumSize() {
        return _minimumSize;
    }

    public int getPrecision() {
        return _precision;
    }

    public abstract void moveControlPoint(int controlPoint, Point destinationPoint);

    public void setMinimumSize(int minimumSize) {
        _minimumSize = minimumSize;
    }

    public void setPrecision(int precision) {
        _precision = precision;
    }
}
