/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.layout;

import java.util.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public final class ActorEventBroadcasterImpl implements ActorEventBroadcaster {
    private TreeSet _listeners;
    private ActorEvent _actorEvent;

    private class listenerComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            int hashCode1 = o1.hashCode();
            int hashCode2 = o2.hashCode();
            if (hashCode1 < hashCode2) {
                return -1;
            } else {
                if (hashCode1 > hashCode2) {
                    return 1;
                }
            }
            return 0;
        }
    }

    public ActorEventBroadcasterImpl(Actor actor) {
        _actorEvent = new ActorEvent(actor);
        _listeners = new TreeSet(new listenerComparator());
    }

    public void addActorListener(ActorListener actorListener) {
        if (!_listeners.contains(actorListener)) {
            _listeners.add(actorListener);
        }
    }

    public void broadcastActorDoubleClickedEvent() {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ActorListener) i.next()).actorDoubleClicked(_actorEvent);
        }
    }

    public void broadcastActorMovedEvent() {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ActorListener) i.next()).actorMoved(_actorEvent);
        }
    }

    public void broadcastActorResizedEvent() {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ActorListener) i.next()).actorResized(_actorEvent);
        }
    }

    public void broadcastActorSelectedEvent() {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ActorListener) i.next()).actorSelected(_actorEvent);
        }
    }

    public void broadcastActorStartedMovingEvent() {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ActorListener) i.next()).actorStartedMoving(_actorEvent);
        }
    }

    public void broadcastActorStoppedMovingEvent() {
        Iterator i = _listeners.iterator();
        while (i.hasNext()) {
            ((ActorListener) i.next()).actorStoppedMoving(_actorEvent);
        }
    }

    public void removeActorListener(ActorListener actorListener) {
        _listeners.remove(actorListener);
    }
}
