/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.layout;

import java.awt.*;

/**
 *  Description of the Class Tiles an image for the background
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class CleanSweepBackgroundManager implements BackgroundManager {
    private AnimationContext _animationContext;
    private Image _tile;
    private Image _background;
    private Rectangle _bounds;

    public CleanSweepBackgroundManager(AnimationContext animationContext, Image tile) {
        _animationContext = animationContext;
        _tile = tile;
        _background = null;
    }

    // the BackgroundManager API
    public void addDirtyRect(Rectangle dirtyRect) {
        // no-op
    }

    public void backgroundHasChangedSize() {
        _background = null;
    }

    public void drawEntireBackground(Graphics g) {
        g.drawImage(_background, _bounds.x, _bounds.y, null);
    }

    public void refreshBackground(Graphics g) {
        if (null == _background) {
            retile();
        }
        g.drawImage(_background, _bounds.x, _bounds.y, null);
    }

    private void retile() {
        _bounds = _animationContext.getCoordinateSystem();
        _background = TiledImage.createTiledImage(_tile, _bounds, _animationContext.getOwner());
    }

    // And some extra image maintenance stuff
    public void setBackgroundTile(Image tile) {
        _tile = tile;
        _background = null;
    }
}
