/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.layout;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class Line extends Observable {

    public Point startingPoint = new Point();
    public Point endingPoint = new Point();
    public final static int SMALL_AMOUNT = 2;
    // hit testing amoung
    private final static String ABOVE = "above";
    // not used
    private final static String BELOW = "below";

    private void announceChanges() {
        setChanged();
        notifyObservers();
    }

    private boolean arePointsEqual(Point firstPoint, Point secondPoint) {
        if (null == firstPoint) {
            return (null == secondPoint);
        }
        return firstPoint.equals(secondPoint);
    }

    private boolean closeTogether(Point firstPoint, Point secondPoint) {
        if (firstPoint.x - secondPoint.x > SMALL_AMOUNT) {
            return false;
        }
        if (firstPoint.y - secondPoint.y > SMALL_AMOUNT) {
            return false;
        }
        if (secondPoint.x - firstPoint.x > SMALL_AMOUNT) {
            return false;
        }
        if (secondPoint.y - firstPoint.y > SMALL_AMOUNT) {
            return false;
        }
        return true;
    }

    public boolean equals(Object object) {
        if (object instanceof Line) {
            Line line = (Line) object;
            if ((arePointsEqual(startingPoint, line.startingPoint)) && (arePointsEqual(endingPoint, line.endingPoint))) {
                return true;
            }
        }
        return false;
    }

    public Point getIntersection(Line otherLine) {
        if (intersects(otherLine)) {
            return getIntersection(otherLine, startingPoint, endingPoint);
        }
        return null;
    }

    private Point getIntersection(Line otherLine, Point startingPoint, Point endingPoint) {
        Point intermediatePoint = new Point();
        intermediatePoint.x = (startingPoint.x + endingPoint.x) / 2;
        intermediatePoint.y = (startingPoint.y + endingPoint.y) / 2;
        if (closeTogether(startingPoint, endingPoint)) {
            return intermediatePoint;
        }
        if (Line2D
            .linesIntersect(
                startingPoint.x,
                startingPoint.y,
                intermediatePoint.x,
                intermediatePoint.y,
                otherLine.startingPoint.x,
                otherLine.startingPoint.y,
                otherLine.endingPoint.x,
                otherLine.endingPoint.y)) {
            return getIntersection(otherLine, startingPoint, intermediatePoint);
        }
        return getIntersection(otherLine, intermediatePoint, endingPoint);
    }

    public boolean intersects(Line otherLine) {
        return Line2D.linesIntersect(
            startingPoint.x,
            startingPoint.y,
            endingPoint.x,
            endingPoint.y,
            otherLine.startingPoint.x,
            otherLine.startingPoint.y,
            otherLine.endingPoint.x,
            otherLine.endingPoint.y);
    }
}
