/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.layout;

import java.awt.*;
import java.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class Polygon extends AbstractShape {
    protected int[] xPoints;
    protected int[] yPoints;
    protected ArrayList _intersectionPoints = new ArrayList();
    private Line _testLine = new Line();

    public Polygon() {
    }

    protected abstract void buildPolygon(java.awt.Rectangle location);

    private void computeIntersectionPoints(Line line) {
        _intersectionPoints.clear();
        _testLine.startingPoint.x = xPoints[xPoints.length - 1];
        _testLine.startingPoint.y = yPoints[xPoints.length - 1];
        for (int counter = 0; counter < xPoints.length; counter++) {
            _testLine.endingPoint.x = xPoints[counter];
            _testLine.endingPoint.y = yPoints[counter];
            Point intersectionPoint = line.getIntersection(_testLine);
            if (null != intersectionPoint) {
                _intersectionPoints.add(intersectionPoint);
            }
            _testLine.startingPoint.x = xPoints[counter];
            _testLine.startingPoint.y = yPoints[counter];
        }
        return;
    }

    public void drawBorder(Graphics g, java.awt.Rectangle location) {
        g.drawPolygon(xPoints, yPoints, xPoints.length);
    }

    public void drawFilledShape(Graphics g, java.awt.Rectangle location) {
        buildPolygon(location);
        g.fillPolygon(xPoints, yPoints, xPoints.length);
    }

    public void drawShape(Graphics g, java.awt.Rectangle location) {
        buildPolygon(location);
        g.drawPolygon(xPoints, yPoints, xPoints.length);
    }

    public Point getBoundaryPointOnLine(Line line, java.awt.Rectangle shapeLocation) {
        buildPolygon(shapeLocation);
        computeIntersectionPoints(line);
        if (0 == _intersectionPoints.size()) {
            return null;
        }
        return (Point) _intersectionPoints.get(0);
    }

    public boolean intersectsLineUniquely(Line line, java.awt.Rectangle shapeLocation) {
        buildPolygon(shapeLocation);
        computeIntersectionPoints(line);
        return (1 == _intersectionPoints.size());
    }
}
