/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.storage.rdf;

import java.util.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.util.*;


public class NamespaceChanger implements ModelFilter{

  String _oldNamespace;
  String _newNamespace;

  public NamespaceChanger(String oldNamespace, String newNamespace) {
    _oldNamespace = oldNamespace;
    _newNamespace = newNamespace;

  }

  public Collection getStatements(Statement statement, 
      NodeFactory nodeFactory) throws ModelException {
    Vector statements = new Vector(1);
    statements.add(getStatement(statement, nodeFactory));
    return statements;
  }

  Statement getStatement(Statement statement, NodeFactory nodeFactory) 
      throws ModelException {
    Resource oldSubject = statement.subject();
    Resource oldPredicate = statement.predicate();
    RDFNode oldObject = statement.object();
    Resource subject = getResource(oldSubject, nodeFactory);
    Resource predicate = getResource(oldPredicate, nodeFactory);
    RDFNode object = getNode(oldObject, nodeFactory);
    if (oldSubject != subject || oldPredicate != predicate ||
        oldObject != object) // something has changed
      return nodeFactory.createStatement(subject, predicate, object);
    else
      return statement;
  }

  RDFNode getNode(RDFNode node, NodeFactory nodeFactory) throws ModelException {
    if (node instanceof Resource)
      return getResource((Resource)node, nodeFactory);
    else
      return node;
  }

  Resource getResource(Resource resource, NodeFactory nodeFactory) 
      throws ModelException {
    if (resource instanceof Statement)
      // does not terminate for recursive statements ... !!!
      return getStatement((Statement)resource, nodeFactory);
    else if (resource instanceof Model)
      return resource;
    else { // "normal" resource
      if (_oldNamespace.equals(resource.getNamespace()))
        return nodeFactory.createResource(_newNamespace,
          resource.getLocalName());
      else
        return resource;
    }
  }


}


