/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.storage.rdf;

import java.util.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.storage.walker.*;


public class RDFSlotRestriction implements WalkerSlotRestriction {

  WalkerFrame allowedClass;
  Collection allowedClasses;

  RDFSlotRestriction() { // case 1: Literal=String
    allowedClass = null;
  }

  RDFSlotRestriction(WalkerFrame range) { // case 2: range
    allowedClass = range;
    allowedClasses = CollectionUtilities.createCollection(range);
  }

  public boolean isAny() { return false; } // ... ???
  public boolean isBoolean() { return false; }
  public boolean isClass() { return false; } // ... !!!
  public boolean isFloat() { return false; }
  public boolean isInstance() { return allowedClass != null; }
  public boolean isInteger() { return false; }
  public boolean isString() { return allowedClass == null; }
  public boolean isSymbol() { return false; }

  public WalkerFrame getAllowedClass() { return allowedClass; }
  public Collection getAllowedClasses() { return allowedClasses; }
  public Collection getAllowedParents() { return null; }
  public Collection getAllowedValues() { return null; }

  public Collection getValues() { return null; }

  public Collection getDefaultValues() { return null; }

  public int getMinimumCardinality() { return 0; }
  public int getMaximumCardinality() { 
    return KnowledgeBase.MAXIMUM_CARDINALITY_UNBOUNDED;
  }

  public String getMinimumValue() { return null; }
  public String getMaximumValue() { return null; }

  public Map getFacetOverrides() { // FACET
    return null;
  }

}


