/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.storage.rdf.configurable;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.storage.walker.*;
import edu.stanford.smi.protegex.storage.walker.protege.*;
import edu.stanford.smi.protegex.storage.rdf.*;
import edu.stanford.smi.protegex.storage.rdf.roundtrip.*;


public class RDFCBackend extends RDFBackend {

  static final String USE_ROUNDTRIP_PROPERTY = "use_roundtrip";

  public String getDescription() {
    return "RDF Schema";
  }

  public KnowledgeBaseSourcesEditor createKnowledgeBaseSourcesEditor(
      String projectName, PropertyList sources) {
    return new RDFCSourcesEditor(projectName, sources);
  }

  // configuration

  static boolean getUseRoundtripBackend(PropertyList sources) {
    Boolean useRT = sources.getBoolean(USE_ROUNDTRIP_PROPERTY);
    return useRT == null || useRT.booleanValue();
    // default is true!
  }

  public static void saveConfiguration(PropertyList sources, 
      boolean useRoundtripBackend) {
    sources.setBoolean(USE_ROUNDTRIP_PROPERTY, useRoundtripBackend);
    // add here for additional configurations ...
  }


  // load and save

  public void loadWalk(KnowledgeBase kb, PropertyList sources,
      String classesFileName, String instancesFileName, 
      String namespace, Namespaces namespaces,
      Collection errors, boolean included) {
    if (getUseRoundtripBackend(sources))
      new RDFRTFrameWalker(classesFileName, instancesFileName, 
	  namespace, namespaces)
	.walk(new ProtegeFrameCreator(kb, namespaces, included, errors));
    else // simple backend
      // we probably should ALWAYS use the roundtrip backend ... !!!
      new RDFFrameWalker(classesFileName, instancesFileName,
	namespace, namespaces)
	  .walk(new ProtegeFrameCreator(kb, namespaces, included, errors));
  }

  public void saveWalk(KnowledgeBase kb, PropertyList sources,
      String classesFileName, String instancesFileName, 
      Namespaces namespaces, Collection errors) {
    if (kb instanceof RDFKnowledgeBase)
      ((RDFKnowledgeBase)kb).useRenamingNamespaces();
    if (getUseRoundtripBackend(sources))
      new ProtegeFrameWalker(kb, namespaces)
	.walk(new RDFRTFrameCreator(classesFileName,
	  instancesFileName, namespaces, errors));
    else
      new ProtegeFrameWalker(kb, namespaces)
	.walk(new RDFFrameCreator(classesFileName,
	  instancesFileName, namespaces, errors));
  }


}


