/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.storage.rdf.roundtrip;

import java.io.*;
import java.util.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.storage.walker.*;
import edu.stanford.smi.protegex.storage.walker.protege.*;
import edu.stanford.smi.protegex.storage.rdf.*;
// #RV
import org.xml.sax.InputSource;
// #RV

public class RDFRTBackend extends RDFBackend {


  public String getDescription() {
    return "RDFS (round trip: plain Protege / extended RDFS)";
  }

  public void loadWalk(KnowledgeBase kb, PropertyList sources,
      String classesFileName, String instancesFileName, 
      String namespace, Namespaces namespaces,
      Collection errors, boolean included) {
    new RDFRTFrameWalker(classesFileName, instancesFileName, 
	namespace, namespaces)
      .walk(new ProtegeFrameCreator(kb, namespaces, included, errors));
  }

// #RV
  public void loadWalk(KnowledgeBase kb, PropertyList sources,
      InputSource classes, InputSource instances,
      String namespace, Namespaces namespaces,
      Collection errors) {
    new RDFRTFrameWalker(classes, instances,
        namespace, namespaces)
      .walk(new ProtegeFrameCreator(kb, namespaces, false, errors));
  }
// #RV

  public void saveWalk(KnowledgeBase kb, PropertyList sources,
      String classesFileName, String instancesFileName, 
      Namespaces namespaces, Collection errors) {
    new ProtegeFrameWalker(kb, namespaces)
      .walk(new RDFRTFrameCreator(classesFileName,
	instancesFileName, namespaces, errors));
  }

// #RV
  public void saveWalk(KnowledgeBase kb, PropertyList sources,
      Writer rdfsModel, Writer rdfModel,
      Namespaces namespaces, Collection errors) {
    new ProtegeFrameWalker(kb, namespaces)
      .walk(new RDFRTFrameCreator(rdfsModel,
      rdfModel, namespaces, errors));
  }
// #RV

}


