/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.abstracttable;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.*;
import edu.stanford.smi.protege.model.Frame;

/**
 *  Renderer for everything Note: it uses AbstractTableWidgetValue, but only
 *  really uses the color and value slots.
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class GenericTableCellRenderer extends JTextField implements TableCellRenderer {
    private boolean _highlightSelectedRow;
    private Color _treeHighlightingColor;

    public GenericTableCellRenderer(boolean highlightSelectedRow) {
        super();
        _highlightSelectedRow = highlightSelectedRow;
        setBorder(null);
        _treeHighlightingColor = UIManager.getColor("Tree.selectionBackground");
    }

    private String buildString(AbstractTableWidgetValue value) {
        String returnValue = null;
        if (value.values == null) {
            return null;
        }
        Iterator i = (value.values).iterator();
        while (i.hasNext()) {
            Object nextValue = i.next();
            if (null == returnValue) {
                returnValue = getStringForObject(nextValue);
            } else {
                returnValue += ", " + getStringForObject(nextValue);
            }
        }
        return returnValue;
    }

    private String getStringForObject(Object value) {
        String firstPass;
        if (value instanceof Frame) {
            firstPass = ((Frame) value).getBrowserText();
        } else {
            firstPass = value.toString();
        }
        return modifyToHandleCarriageReturns(firstPass);
    }

    public Component getTableCellRendererComponent(
        JTable table,
        Object value,
        boolean isSelected,
        boolean hasFocus,
        int row,
        int column) {
        if ((hasFocus) && (table.isEditing())) {
            table.editingCanceled(new ChangeEvent(this));
        }
        AbstractTableWidgetValue itv = (AbstractTableWidgetValue) (value);
        if (isSelected) {
            setBackground(_treeHighlightingColor);
        } else {
            if (row == table.getSelectedRow() && (_highlightSelectedRow)) {
                setBackground(_treeHighlightingColor);
            } else {
                setBackground(Color.white);
            }
        }
        setForeground(itv.color);
        setText(buildString(itv));
        return this;
    }

    private String modifyToHandleCarriageReturns(String string) {
        int index = string.indexOf('\n');
        if (index < 0) {
            return string;
        }
        return string.substring(0, index) + "...";
    }
}
