/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import java.util.Stack;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class AcyclicPath {

    private boolean[] _inPath;
    private Stack _order;
    private Integer _head;

    public AcyclicPath(int numNodes) {
        this(numNodes, 0);
    }

    public AcyclicPath(int numNodes, int root) {
        initPath(numNodes, root);
        _head = new Integer(root);
        _order = new Stack();
        _order.push(_head);
    }

    public boolean addEdgeTo(int x) {
        if (isInPath(x)) {
            return false;
        } else {
            _inPath[x] = true;
            Integer newTail = new Integer(x);
            _order.push(newTail);
            return true;
        }
    }

    public int getHead() {
        return _head.intValue();
    }

    public int getTail() {
        return ((Integer) (_order.peek())).intValue();
    }

    private void initPath(int numNodes, int root) {
        _inPath = new boolean[numNodes];
        for (int i = 0; i < numNodes; i++) {
            _inPath[i] = (i == root);
        }
        // Mark the root as in the path.
    }

    public boolean isInPath(int x) {
        return _inPath[x];
    }

    public boolean removeEdge() {
        if (getHead() == getTail()) {
            return false;
        } else {
            int x = ((Integer) (_order.pop())).intValue();
            _inPath[x] = false;
            return true;
        }
    }
}
