/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import java.awt.Point;
import java.util.*;

class BaryElem implements Comparable {

    private int _node;
    private double _baryVal;
    private FlowchartGraph _graph;

    public final static int UP = -1;
    public final static int DOWN = +1;
    public final static double MAX = java.lang.Double.MAX_VALUE;

    public BaryElem(int n, int dir, FlowchartGraph g) {
        _graph = g;
        _node = n;
        initBaryVal(dir);
    }

    public int compareTo(Object o) {
        BaryElem be = (BaryElem) (o);
        double diff = getBaryVal() - be.getBaryVal();
        if (diff < 0) {
            return -1;
        } else if (diff > 0) {
            return +1;
        } else {
            return 0;
        }
    }

    public double getBaryVal() {
        return _baryVal;
    }

    public double getLocation() {
        return _graph.getLocationOf(_node);
    }

    public int getNode() {
        return _node;
    }

    private void initBaryVal(int dir) {
        int connectivity = 0;
        _baryVal = 0;
        int depth = _graph.getDepthOf(_node);
        int[] comparisonRow = _graph.getRow(depth + dir);
        for (int i = 0; i < comparisonRow.length; i++) {
            if (_graph.hasEdgeBetween(_node, comparisonRow[i])) {
                _baryVal += i;
                connectivity++;
            }
        }
        _baryVal = (connectivity > 0) ? _baryVal / connectivity : MAX;
    }
}
