/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protege.model.*;
import java.util.*;

/**
 *  This is aggregation and abstraction in an inner loop (the paint loop). We
 *  may need to wrap a cache around these guys for performance. "True" means "I
 *  have no objection to it being displayed"
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class BaseVisibilityChecker extends Observable implements VisibilityChecker {

    protected void announceChanges() {
        setChanged();
        notifyObservers();
    }

    public abstract boolean isConnectorClsVisible(Cls connectorCls);

    public abstract boolean isConnectorVisible(Instance connectorInstance);

    public abstract boolean isNodeClsVisible(Cls nodeCls);

    public abstract boolean isNodeVisible(Instance nodeInstance);
}
