/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protege.model.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.text.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.table.*;
import javax.swing.border.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class ColorRenderer extends JLabel implements TableCellRenderer {
    protected Border _unselectedBorder = null;
    protected Border _selectedBorder = null;
    protected boolean _isBordered = true;
    protected DiagramWidgetState _state;

    protected ImageIcon _imageIcon;
    protected Image _image;
    protected int _height;
    protected int _width;

    public ColorRenderer(boolean isBordered, JTable table, DiagramWidget widget) {
        super();
        _isBordered = isBordered;
        _state = widget.getState();
        setOpaque(true);
        _unselectedBorder = BorderFactory.createMatteBorder(2, 5, 2, 5, table.getBackground());
        _selectedBorder = BorderFactory.createMatteBorder(2, 5, 2, 5, table.getSelectionBackground());
    }

    protected abstract void drawInImage(Cls cls);

    public Component getTableCellRendererComponent(
        JTable table,
        Object color,
        boolean isSelected,
        boolean hasFocus,
        int row,
        int column) {
        TableModel model = table.getModel();
        setBackground(Color.white);

        Rectangle boundary = table.getCellRect(row, column, false);

        if ((null == _imageIcon) || (_height != boundary.height) || (_width != boundary.width)) {
            _height = boundary.height;
            _width = boundary.width;
            _image = table.createImage(_width, _height);
            _imageIcon = new ImageIcon(_image);
            setIcon(_imageIcon);
        }
        Cls cls = (Cls) (model.getValueAt(row, column));
        drawInImage(cls);
        if (_isBordered) {
            if (isSelected) {
                setBorder(_selectedBorder);
            } else {
                setBorder(_unselectedBorder);
            }
        }
        return this;
    }
}
