/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public final class DiagramIcons {
    private static Map theIcons = new HashMap();

    public static Icon getEmptyIcon() {
        return getIcon("Empty");
    }

    public static Icon getFlowchartIcon() {
        //PM
        return getIcon("Flowchart");
    }

    public static Icon getFrameHiderIcon() {
        return getIcon("Broom");
    }

    private static Icon getIcon(String name) {
        Icon icon = (Icon) theIcons.get(name);
        if (icon == null || icon.getIconWidth() == -1) {
            icon = loadIcon(name);
            theIcons.put(name, icon);
        }
        return icon;
    }

    public static Icon getLegendIcon() {
        return getIcon("Legend");
    }

    public static Icon getMiniformIcon() {
        return getIcon("Miniform");
    }

    public static Icon getSelectionIcon() {
        return getIcon("Triangle");
    }

    public static Icon getSolderingIron() {
        return getIcon("SolderingIron");
    }

    public static Icon getThumbnailIcon() {
        return getIcon("Thumbnail");
    }

    private static Icon loadIcon(String name) {
        Icon icon = null;
        String path = "image/" + name + ".gif";
        URL url = DiagramIcons.class.getResource(path);
        if (url != null) {
            icon = new ImageIcon(url);
            if (icon.getIconWidth() == -1) {
                Log.error("failed to load", DiagramIcons.class, "loadIcon", name);
            }
        }
        if (icon == null) {
            icon = edu.stanford.smi.protege.resource.Icons.getUglyIcon();
        }
        return icon;
    }
}
