/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;

import edu.stanford.smi.protege.model.*;
import java.awt.*;
import java.util.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class DiagramsAnimationContext extends AnimationContext {
    private Rectangle _selectionRectangle;
    private Rectangle _scratchRectangle;
    private TestForInstance _testForInstance;

    private class TestForInstance implements ActorTest {
        public Instance instance;

        public boolean performTest(Actor target) {
            if (target instanceof ActorInstance) {
                return instance.equals(((ActorInstance) target).getInstance());
            }
            return false;
        }
    }

    public DiagramsAnimationContext(
        boolean visibleToEvents,
        BackgroundManager backgroundManager,
        Rectangle coordinateSystem,
        Component owner,
        AnimationContextMouseEventHandler animationContextMouseEventHandler) {
        super(visibleToEvents, backgroundManager, coordinateSystem, owner, animationContextMouseEventHandler);
        _testForInstance = new TestForInstance();
        _selectionRectangle = new Rectangle();
        _scratchRectangle = new Rectangle();
    }

    private void copyRect(Actor firstActor) {
        (firstActor.getLocation()).getBounds(_selectionRectangle);
        return;
    }

    public void enlargeToHeight(int newHeight) {
        if (_coordinateSystem.height < newHeight) {
            _coordinateSystem.height = newHeight;
        }
    }

    public void enlargeToWidth(int newWidth) {
        if (_coordinateSystem.width < newWidth) {
            _coordinateSystem.width = newWidth;
        }
    }

    private void expandRect(Actor nextActor) {
        (nextActor.getLocation()).getBounds(_scratchRectangle);
        _selectionRectangle.add(_scratchRectangle);
        return;
    }

    public Actor getActorForInstance(Instance instance) {
        _testForInstance.instance = instance;
        Actor actorForInstance = _zOrder.performSingleTest(ZOrder.TOPDOWN, _testForInstance);
        return actorForInstance;
    }

    public Rectangle getSelectionRectangle() {
        Collection actors = getSelectedActors();
        if ((null == actors) || (0 == actors.size())) {
            return null;
        }
        Iterator i = actors.iterator();
        Actor firstActor = (Actor) i.next();
        copyRect(firstActor);
        while (i.hasNext()) {
            Actor nextActor = (Actor) i.next();
            expandRect(nextActor);
        }
        return _selectionRectangle;
    }

    public void selectInstance(Instance instance) {
        _testForInstance.instance = instance;
        Actor actorToSelect = _zOrder.performSingleTest(ZOrder.TOPDOWN, _testForInstance);
        if (null != actorToSelect) {
            selectActor(actorToSelect);
        }
        return;
    }
}
