/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.util.List;

/*
 * Here because, when finding a click's owner, nodes should always
 * win over wires
 */

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class DiagramsAnimationContextMouseEventHandler extends AnimationContextMouseEventHandler {

    public DiagramsAnimationContextMouseEventHandler() {
        super();
    }

    public DiagramsAnimationContextMouseEventHandler(DiagramsAnimationContext animationContext) {
        super(animationContext);
    }

    protected Actor findEventOwner(MouseEvent e) {
        _eventOwnerTest.setEvent(e);
        ZOrder zOrder = _animationContext.getZOrder();
        List possibilities = zOrder.performCollectiveTest(ZOrder.TOPDOWN, _eventOwnerTest);
        Actor returnValue = getFirstNodeFromList(possibilities);
        if (null == returnValue) {
            returnValue = getFirstConnectorFromList(possibilities);
        }
        return returnValue;
    }

    private Actor getFirstConnectorFromList(List possibilities) {
        Iterator i = possibilities.iterator();
        while (i.hasNext()) {
            Actor nextActor = (Actor) i.next();
            if (nextActor instanceof WireActorInstance) {
                return nextActor;
            }
        }
        return null;
    }

    private Actor getFirstNodeFromList(List possibilities) {
        Iterator i = possibilities.iterator();
        while (i.hasNext()) {
            Actor nextActor = (Actor) i.next();
            if (nextActor instanceof GlyphActorInstance) {
                return nextActor;
            }
        }
        return null;
    }
}
