/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;

import javax.swing.*;
import javax.swing.plaf.basic.*;
import javax.swing.plaf.metal.*;
import javax.swing.plaf.ComponentUI.*;
import java.awt.*;
import java.awt.event.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class Diagrams_InternalFrameAction extends Diagrams_Action {
    protected InternalFrame _associatedFrame;

    private class InternalFrameMouseListener extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            if (2 == e.getClickCount()) {
                _widget.removeInternalFrame(_associatedFrame);
            }
        }
    }

    public Diagrams_InternalFrameAction(
        DiagramWidget widget,
        String tooltip,
        Icon icon,
        AnimationContext animationContext) {
        super(widget, animationContext, tooltip, icon);
    }

    public void actionPerformed(ActionEvent event) {
        if (null == _associatedFrame) {
            _associatedFrame = buildAssociatedInternalFrame();
            configureAssociatedFrame();
        }
        displayAssociatedFrame();
    }

    protected abstract InternalFrame buildAssociatedInternalFrame();

    private void configureAssociatedFrame() {
        JComponent titlePane = getTitlePane();
        titlePane.addMouseListener(new InternalFrameMouseListener());
        setFrameToDefaultSize();
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                _associatedFrame.validate();
                _associatedFrame.repaint();
            }
        });
    }

    private void displayAssociatedFrame() {
        _widget.addInternalFrame(_associatedFrame);
        _associatedFrame.setVisible(true);
        _associatedFrame.setSelected(true);
    }

    private JComponent getTitlePane() {
        BasicInternalFrameUI internalFrameUI = (BasicInternalFrameUI) _associatedFrame.getUI();
        return internalFrameUI.getNorthPane();
    }

    protected void setFrameToDefaultSize() {
        _associatedFrame.setSize(200, 200);
    }
}
