/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;
import edu.stanford.smi.protegex.util.*;

import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protege.ui.*;
import edu.stanford.smi.protege.widget.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class Diagrams_PrototypeAction extends Diagrams_AddNode {
    private int _depth;
    private InstanceCopier _instanceCopier;
    private ZOrder _zOrder;
    private AddInstancesLater _addInstancesLater;
    private ArrayList _copies;

    private class InstanceCopier implements ActorAction {
        public void performMethod(Actor actor) {
            if (actor.getIsSelected()) {
                Instance instance = ((ActorInstance) actor).getInstance();
                RecursiveCopy.recursivelyCopyInstance(instance, _depth, _copies);
                SwingUtilities.invokeLater(_addInstancesLater);
            }
        }
    }

    private class AddInstancesLater implements Runnable {
        public void run() {
            Iterator i = _copies.iterator();
            while (i.hasNext()) {
                Instance nextInstance = (Instance) i.next();
                addInstance(nextInstance);
            }
            _copies.clear();
        }
    }

    public Diagrams_PrototypeAction(
        DiagramsPanel panel,
        DiagramWidget widget,
        AnimationContext mainDrawingArea,
        int depth) {
        super(panel, widget, mainDrawingArea, (widget.getState()).getPrototypeButtonTooltip(), Icons.getCopyIcon());
        _depth = depth;
        _instanceCopier = new InstanceCopier();
        _copies = new ArrayList();
        _addInstancesLater = new AddInstancesLater();
    }

    public void actionPerformed(ActionEvent e) {
        _zOrder = _mainDrawingArea.getZOrder();
        _copies.clear();
        _zOrder.performAction(0, _instanceCopier);
    }
}
