/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import java.awt.Point;
import java.util.*;

class LayoutNode implements Comparable {

    private int _depth;
    private double _location;
    private boolean[] _edges;
    private boolean _dummy;

    public final static int UNASSIGNED_DEPTH = -1;
    public final static int ROOT_DEPTH = 0;
    public final static double UNASSIGNED_LOCATION = -42.0;

    public LayoutNode(int graphSize) {
        this(graphSize, false);
    }

    public LayoutNode(int graphSize, LayoutNode oldNode) {
        _edges = new boolean[graphSize];
        for (int i = 0; i < graphSize; i++) {
            _edges[i] = (i < oldNode._edges.length) ? oldNode.hasEdgeTo(i) : false;
        }
        _depth = oldNode.getDepth();
        _location = oldNode.getLocation();
        _dummy = oldNode.isDummyNode();
    }

    public LayoutNode(int graphSize, boolean isDummy) {
        _edges = new boolean[graphSize];
        for (int i = 0; i < graphSize; i++) {
            _edges[i] = false;
        }
        _depth = UNASSIGNED_DEPTH;
        _location = UNASSIGNED_LOCATION;
        _dummy = isDummy;
    }

    public void addEdgeTo(int to) {
        _edges[to] = true;
    }

    public int compareTo(Object o) {
        LayoutNode n = (LayoutNode) (o);
        double diff = getLocation() - n.getLocation();
        if (diff < 0) {
            return -1;
        } else if (diff > 0) {
            return +1;
        } else {
            return 0;
        }
    }

    public int getDepth() {
        return _depth;
    }

    public double getLocation() {
        return _location;
    }

    public boolean hasEdgeTo(int to) {
        return _edges[to];
    }

    public boolean isDummyNode() {
        return _dummy;
    }

    public void setDepth(int depth) {
        _depth = depth;
    }

    public void setLocation(double location) {
        _location = location;
    }

    public String toString() {
        String result = (isDummyNode()) ? "*" : "";
        result = result + "(" + getDepth() + ", " + getLocation() + ")";
        for (int i = 0; i < _edges.length; i++) {
            if (hasEdgeTo(i)) {
                result = result + " " + i;
            }
        }
        return result;
    }
}
