/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import java.awt.*;
import javax.swing.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class Legend extends InternalFrame {

    public Legend(DiagramWidget widget, Diagrams_InternalFrameAction action) {
        super(widget, action, "Legend");
    }

    protected void buildGui() {
        JSplitPane mainPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        mainPanel.setTopComponent(buildNodePanel());
        mainPanel.setBottomComponent(buildWirePanel());
        getContentPane().add(mainPanel);
    }

    private JComponent buildNodePanel() {
        TablePanelModel tablePanelModel = new TablePanelModel((_widget.getState()).getNodeClses());
        JTable table = new JTable(tablePanelModel);
        table.setPreferredScrollableViewportSize(new Dimension(500, 70));
        table.setRowSelectionAllowed(false);
        table.setColumnSelectionAllowed(false);
        table.setCellSelectionEnabled(false);
        (table.getColumnModel().getColumn(1)).setCellRenderer(new NodeColorRenderer(true, table, _widget));
        JScrollPane scrollPane = new JScrollPane(table);
        return scrollPane;
    }

    private JComponent buildWirePanel() {
        TablePanelModel tablePanelModel = new TablePanelModel((_widget.getState()).getConnectorClses());
        JTable table = new JTable(tablePanelModel);
        table.setPreferredScrollableViewportSize(new Dimension(500, 70));
        table.setRowSelectionAllowed(false);
        table.setColumnSelectionAllowed(false);
        table.setCellSelectionEnabled(false);
        (table.getColumnModel().getColumn(1)).setCellRenderer(new WireColorRenderer(true, table, _widget));
        JScrollPane scrollPane = new JScrollPane(table);
        return scrollPane;
    }
}
