/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import javax.swing.*;
import java.awt.*;
import java.util.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class LineDescription extends ThingWithWidth {
    private ArrayList _stringDescriptions;
    private int _widthOfASpace;
    private String _totalString;

    public LineDescription(int widthOfASpace) {
        _widthOfASpace = widthOfASpace;
        _stringDescriptions = new ArrayList();
    }

    public void add(StringDescription stringDescription) {
        _stringDescriptions.add(stringDescription);
        _width += stringDescription.getWidth() + _widthOfASpace;
    }

    public void prepareToRender() {
        Iterator i = _stringDescriptions.iterator();
        _totalString = "";
        boolean prependASpace = false;
        while (i.hasNext()) {
            StringDescription nextDescription = (StringDescription) i.next();
            if (prependASpace) {
                _totalString += " " + nextDescription.getString();
            } else {
                _totalString += nextDescription.getString();
                prependASpace = true;
            }
        }
        return;
    }

    public void render(Graphics g, Point startingPoint) {
        g.drawString(_totalString, startingPoint.x, startingPoint.y);
    }
}
