/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;

import edu.stanford.smi.protege.model.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.text.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.table.*;
import javax.swing.border.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class NodeColorRenderer extends ColorRenderer {
    private Rectangle _location;

    public NodeColorRenderer(boolean isBordered, JTable table, DiagramWidget widget) {
        super(isBordered, table, widget);
        _location = new Rectangle(0, 0, _width, _height);
    }

    protected void drawInImage(Cls cls) {
        NodeState nodeState = _state.getStateForNode(cls);
        Graphics g = _image.getGraphics();
        g.setColor(Color.white);
        g.fillRect(0, 0, _width, _height);
        _location.width = _width;
        _location.height = _height;
        drawNode(nodeState, g, _location);
        g.dispose();
    }

    private void drawNode(NodeState state, Graphics g, Rectangle location) {
        ActorShape shapeToDraw = ActorShapeFactory.getShapeForName(state.getNodeStyle());
        if (null == shapeToDraw) {
            System.err.println("nodeStyle " + state.getNodeStyle() + " is required but undefined.");
            return;
        }
        shapeToDraw.drawShape(g, state.getObjectColor(), state.getNodeFilled(), location);
    }
}
