/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.widget.*;
import edu.stanford.smi.protege.ui.*;
import edu.stanford.smi.protege.util.*;
import java.awt.*;
import java.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class NodeState extends DiagramObjectState {
    protected String _nodeStyle;
    protected boolean _filled;
    protected ArrayList _listOfVisibleSlots;
    protected final static String NODE_STYLE = ":NODE:STYLE";
    protected final static String FILLED_NODE = ":FILLED:NODE";

    public NodeState(PropertyList pList) {
        super(pList);
        _objectColor = readColor(pList, OBJECT_COLOR_RGB_DESCRIPTOR, NodeConstants.DEFAULT_NODE_COLOR);
        _textColor = readColor(pList, TEXT_COLOR_RGB_DESCRIPTOR, NodeConstants.DEFAULT_TEXT_COLOR);
        _displayText = readBoolean(pList, DISPLAY_TEXT, NodeConstants.DEFAULT_DISPLAY_TEXT);
        _nodeStyle = readString(pList, NODE_STYLE, NodeConstants.DEFAULT_NODE_STYLE);
        _filled = readBoolean(pList, FILLED_NODE, NodeConstants.NODES_DEFAULT_TO_FILLED);
    }

    public boolean getNodeFilled() {
        return _filled;
    }

    public String getNodeStyle() {
        return _nodeStyle;
    }

    public void setNodeFilled(boolean filled) {
        _filled = filled;
    }

    public void setNodeStyle(String nodeStyle) {
        _nodeStyle = nodeStyle;
    }

    public void writeToPropertyList(PropertyList pList) {
        super.writeToPropertyList(pList);
        pList.setString(NODE_STYLE, _nodeStyle);
        pList.setBoolean(FILLED_NODE, _filled);
    }
}
