/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;

/**
 *  Description of the Class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class OuterPanel extends JSplitPane {
    protected AnimationContext _overlay;

    protected MainDrawingPanel _leftPanel;
    protected PalettePanel _rightPanel;
    protected DiagramScrollPane _leftScrollPane;
    protected DiagramScrollPane _rightScrollPane;
    private Dimension d;
    private int leftMaxX = 0, leftMaxY = 0, rightMaxX = 0, rightMaxY = 0;
    private boolean _hasBeenPainted;

    private class SplitPaneListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if ((evt.getPropertyName()).equals(LAST_DIVIDER_LOCATION_PROPERTY)) {
                resetSubpanelLocations(getDividerLocation());
            }
        }
    }

    public OuterPanel() {
        super(JSplitPane.HORIZONTAL_SPLIT, false);
        addPropertyChangeListener(new SplitPaneListener());
        enableEvents(AWTEvent.MOUSE_EVENT_MASK | AWTEvent.MOUSE_MOTION_EVENT_MASK);
    }

    public void paint(Graphics g) {
        if (!_hasBeenPainted) {
            resetSubpanelLocations(getDividerLocation());
            _hasBeenPainted = true;
        }
        super.paint(g);
        if (null != _overlay) {
            _overlay.paint(g);
        }
    }

    protected void processMouseEvent(MouseEvent mouseEvent) {
        Point locationOfEvent = mouseEvent.getPoint();
        if ((_leftScrollPane.contains(locationOfEvent)) && (_leftPanel.wantsEventAtLocation(locationOfEvent))) {
            _leftPanel.forwardMouseEvent(mouseEvent);
        } else {
            if (_rightPanel.wantsEventAtLocation(locationOfEvent)) {
                _rightPanel.forwardMouseEvent(mouseEvent);
            } else {
                _overlay.processMouseEvent(mouseEvent);
            }
        }
        repaint();
    }

    protected void processMouseMotionEvent(MouseEvent mouseEvent) {
        Point locationOfEvent = mouseEvent.getPoint();
        if ((_leftScrollPane.contains(locationOfEvent)) && (_leftPanel.wantsEventAtLocation(locationOfEvent))) {
            _leftPanel.forwardMouseMotionEvent(mouseEvent);
        } else {
            if (_rightPanel.wantsEventAtLocation(locationOfEvent)) {
                _rightPanel.forwardMouseMotionEvent(mouseEvent);
            } else {
                _overlay.processMouseMotionEvent(mouseEvent);
            }
        }
        repaint();
    }

    public void resetSubpanelLocations(int dividerLocation) {
        Insets insets = getInsets();
        if (null != _leftPanel) {
            _leftPanel.setLeftOffset(insets.left);
            _leftPanel.setVisibleWidth(dividerLocation - insets.left);
            if (null != _rightPanel) {
                _rightPanel.setLeftOffset(dividerLocation + getDividerSize());
            }
        }
    }

    public void setDividerLocation(int location) {
        super.setDividerLocation(location);
        resetSubpanelLocations(location);
    }

    public void setLeftHandSide(
        AnimationContext leftHandSide,
        MainDrawingPanel leftPanel,
        DiagramScrollPane leftScrollPane) {
        setLeftComponent(leftScrollPane);
        _leftPanel = leftPanel;
        _leftScrollPane = leftScrollPane;
    }

    public void setOverlay(AnimationContext overlay) {
        _overlay = overlay;
    }

    public void setRightHandSide(
        AnimationContext rightHandSide,
        PalettePanel rightPanel,
        DiagramScrollPane rightScrollPane) {
        setRightComponent(rightScrollPane);
        _rightPanel = rightPanel;
        _rightScrollPane = rightScrollPane;
    }

    public void translateActorFromOverlayIntoMainDrawingArea(Actor actor) {
        actor.translate(
            (_leftScrollPane.getHorizontalScrollBar()).getValue(),
            (_leftScrollPane.getVerticalScrollBar()).getValue());
    }

    public void translateActorIntoOverlayFromPalette(Actor actor) {
        actor.translate(_leftScrollPane.getWidth() + getDividerSize(), 0);
    }
}
