/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import java.awt.Point;
import java.util.*;

class PRElem implements Comparable {

    private int _node;
    private double _priority;
    private double _position;
    private FlowchartGraph _graph;

    public final static int MAX = java.lang.Integer.MAX_VALUE;

    public PRElem(int n, FlowchartGraph g) {
        _graph = g;
        _node = n;
        initPositionAndPriority();
    }

    public int compareTo(Object o) {
        PRElem pe = (PRElem) (o);
        double diff = pe.getPriority() - getPriority();
        if (diff > 0) {
            return +1;
        } else if (diff < 0) {
            return -1;
        } else {
            return 0;
        }
    }

    public int getNode() {
        return _node;
    }

    public double getPreferredPosition() {
        return _position;
    }

    public double getPriority() {
        return _priority;
    }

    private void initPositionAndPriority() {
        _priority = 0.0;
        _position = 0.0;
        int depth = _graph.getDepthOf(_node);
        int[] comparisonRow = _graph.getUnsortedRow(depth - 1);
        for (int i = 0; i < comparisonRow.length; i++) {
            if (_graph.hasEdgeBetween(_node, comparisonRow[i])) {
                _priority++;
                _position += _graph.getLocationOf(comparisonRow[i]);
            }
        }
        _position /= _priority;
        if (_graph.isDummyNode(_node)) {
            _priority = MAX;
        }
        // Give central nodes higher priority.
        double pos = _graph.getLocationOf(_node);
        int rowLength = _graph.getSizeOfRow(depth);
        double mid = (rowLength - 1) / (2.0);
        double distance = (mid > pos) ? mid - pos : pos - mid;
        _priority -= distance / rowLength;
    }

    public void setPosition(double pos) {
        _position = pos;
    }
}
