/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;

import edu.stanford.smi.protege.model.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class RootListModel extends AbstractListModel {

    private DiagramWidget _widget;
    private Object[] _allNodes;

    public RootListModel(DiagramWidget widget) {
        _widget = widget;
        refreshNodes();
    }

    public Object getElementAt(int index) {
        return getTextAt(index);
    }

    public int getIndexOfInstance(Instance ins) {
        for (int i = 0; i < getSize(); i++) {
            if (getInstanceAt(i).equals(ins)) {
                return i;
            }
        }
        return -1;
    }

    public Instance getInstanceAt(int index) {
        return (Instance) (_allNodes[index]);
    }

    public int getSize() {
        return _allNodes.length;
    }

    public String getTextAt(int index) {
        return getTextFor(getInstanceAt(index));
    }

    public static String getTextFor(Instance ins) {
        String result = ins.getBrowserText();
        if (result == null || result.equals("")) {
            result = ins.getName();
        }
        return result;
    }

    public void refreshNodes() {
        _allNodes = _widget.getNodes().toArray();
        Arrays.sort(_allNodes, new RootListCompare());
    }
}
