/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.diagram;

import edu.stanford.smi.protegex.layout.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.text.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.table.*;
import javax.swing.border.*;

/**
 *  Description of the class
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public abstract class TablePanel extends JPanel {
    protected JTable _table;
    protected JPanel _editingPanel;
    protected DiagramWidget _widget;
    protected DiagramWidgetState _state;
    protected TablePanelModel _tablePanelModel;

    private class SelectionListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            resetEditingPanel();
        }
    }

    public TablePanel(Collection clses, DiagramWidget widget) {
        super();
        _widget = widget;
        _state = widget.getState();
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        _tablePanelModel = new TablePanelModel(clses);
        _table = new JTable(_tablePanelModel);
        _table.setPreferredScrollableViewportSize(new Dimension(500, 70));
        _table.setRowSelectionInterval(0, 0);

        JScrollPane scrollPane = new JScrollPane(_table);
        this.add(scrollPane);
        _editingPanel = getEditingPanel();
        this.add(_editingPanel);
        resetEditingPanel();

        _table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        ListSelectionModel rowSM = _table.getSelectionModel();
        setUpColorRenderer(_table.getColumnModel().getColumn(1));
        rowSM.setSelectionInterval(0, 0);
        rowSM.addListSelectionListener(new SelectionListener());
    }

    protected abstract JPanel getEditingPanel();

    protected abstract void resetEditingPanel();

    protected abstract void setUpColorRenderer(TableColumn column);
}
