/*
$Log: TestSearchSynonyme.java,v $
Revision 1.1  2002/08/13 10:39:07  hatinen
Added more functional tests

*/
package fi.helsinki.cs.keltsi.functest;

import com.meterware.httpunit.*;

//import fi.helsinki.cs.keltsi.jenardf.*;
//import fi.helsinki.cs.keltsi.util.*;
import java.io.IOException;
import java.net.MalformedURLException;

import org.xml.sax.*;
import org.w3c.dom.*;

import junit.framework.*;

import java.util.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.common.*;

/**
 * Acceptance test for Searching synonyme classes.
 * http://pharazon.avatars.fi:3000/index.php/Synonyymipalveluluokat
 * @Author $Author: hatinen $
 * @Version $Revision: 1.1 $ $Date: 2002/08/13 10:39:07 $
 */
public class TestSearchSynonyme extends TestCase {

    public static void main(String args[]) {
        junit.textui.TestRunner.run( suite() );
    }

    public static Test suite() {
        return new TestSuite( TestSearchSynonyme.class );
    }

    public TestSearchSynonyme(String name) {
        super(name);
    }

    protected static final String SERVER_NAME = null;
        //"http://db.cs.helsinki.fi:5123/keltsi/";
    protected static final String INDEX = "index.jsp";

    protected WebConversation wc = null;
    protected WebRequest request = null;
    protected WebResponse response = null;

    /**
     * initialize the test
     */
    protected void setUp() throws   java.net.MalformedURLException,
                                    java.io.IOException,
                                    org.xml.sax.SAXException
    {

        this.wc = new WebConversation();
        this.request = new GetMethodWebRequest(SERVER_NAME+INDEX);
        this.response = wc.getResponse(request);
        if (wc == null) fail("init() failure: WebConversation is null");
        if (response == null) fail("init() failure: response is null");
        if (request == null) fail("init() failure: request is null");
    }

    /**
     * Submit search-frame's form and see if the other frame changes.
     */
    public void testSearchService() throws Exception  {
        if (SERVER_NAME == null) return;
        if (wc == null) fail("wc is null");

        WebResponse search = wc.getFrameContents("search");
        WebResponse firstOntology = wc.getFrameContents("ontology");

        if (search == null) fail("search - frame not found");
        if (firstOntology == null) fail("ontology -frame not found");
        //check if clicking service changes the ontology-frame

        //submit form
        WebRequest submitRequest = search.getForms()[0].getRequest();

        request.setParameter("component-select", "Service");
        WebResponse ontology = wc.getResponse(request);

        //check if original ontology-tree and one we just got are different
        assertTrue(!ontology.equals(firstOntology));
    }

    /**
     * Search for instances that have synonym -semantic links
     */
    public void testSearchFranchiseeOf() throws Exception  {
        if (SERVER_NAME == null) return;
        if (wc == null) fail("wc is null");
        WebResponse ontology = wc.getFrameContents("ontology");

        //click following links:
        String[] steps = {
            "Service",
            "PersonalAndDomesticServices",
            "PersonalAppearance",
            "HairCare",
            "HairCuttingOrColorServices"
        };
        System.out.println("trying steps");

        for (int i = 0 ; i < steps.length ; i++) {
            WebLink link = ontology.getLinkWith(steps[i]);
            if (link == null ) {
                 fail("Defined steps (clicks) lead to null");
                 return;
            }
            /*
            System.out.println("Link: "+link.getURLString() +
                " step: "+steps[i]);
            */
            WebRequest r = link.getRequest();
            //System.out.println("r: "+r);
            if (r == null) fail("next request was null");
            ontology = wc.getResponse(r);
            //System.out.println("ontology: "+ontology);
            if (ontology == null) fail("ontology was null");
        }
        System.out.println("steps ok");
        //hope that we have now instances of Hairdressers in ontology-frame
        //click instances and find one with franchisee-of -property
        WebResponse detail = wc.getFrameContents("detail");
        WebResponse originalDetail = detail;
        WebTable table = ontology.getTableWithID("instances");
        if (table == null) fail("table shouldn't be null");
        int found = 0;
        for (int i = 0; i < table.getRowCount(); i++) {
            WebLink link = table.getTableCell(i,0).getLinks()[0];
            if (link == null) fail("link shouldn't be null");
            //open details
            detail = wc.getResponse(link.getRequest());
            if (detail == null) fail("detail shoudn't be null");
            //System.out.println("baa");
            // hope that details have changed
            assertTrue(!detail.equals(originalDetail));
            // get links
            WebResponse links = wc.getFrameContents("detail:links");
            if (links == null) fail("links was null");
            // search for franchisee-of
            WebTable linkTable = links.getTables()[0];
            for (int j=0;j<linkTable.getRowCount();j++) {
                if (linkTable.getTableCell(j,0).asText()
                    .compareTo("synonym") == 0) found++;
            }
        }
        Assert.assertTrue("Should have found links but didn't:"+
                " found = "+found,found>0);
    }

}



