package fi.helsinki.cs.keltsi.tags;

import fi.helsinki.cs.keltsi.util.*;
import fi.helsinki.cs.keltsi.jenardf.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.common.*;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.JspException;
import java.util.*;
import javax.servlet.jsp.PageContext;

/** 
 * This tag handler displays contact information of the
 * assigned service and the name of the service provider
 * (advertiser). This is done by searching the ontology
 * for values of the specified properties.
 * 
 *  Author Tuomo Kajava
 */
public class DisplayAdvertisement extends TagSupport {
    private final static String imagePath="img/";
    private String serviceUri;
    private KeltsiDAML model;
    private Html html;
    private Link link;

    public int doStartTag() throws JspException {
        String advString = "";
        html = new Html(pageContext.getOut());
        html.write("<!-- DisplayAdvertisement starting -->\n");

        /* Get the model from scope. */
        try {
            model = (KeltsiDAML) pageContext.findAttribute(
                    "fi.helsinki.cs.keltsi.KeltsiDAML");
        } catch (Exception ex) {
            html.errorMsg(ex, "while getting the model from scope");
        }
        Service service=null;
        try {
            HttpServletRequest req =
                    (HttpServletRequest)pageContext.getRequest();
            // Get serviceuri.
            serviceUri = req.getParameter("serviceuri");
            // Get service object.
            service=(Service) model.getKeltsiResource(serviceUri);
            link = new Link(req);
        } catch(Exception ex) {
            html.errorMsg(ex, " DA 1");
        }

        Advertisement adv = null;
        try {
            adv = service.getAdvertisement();
            advString = adv.getLabel();
        } catch(Exception ex) {
            html.errorMsg(ex, " DA 2");
        }

        try {
            ImageFile img = adv.getImageFile();
            if (img != null) {
                String imgURL = imagePath + img.path();
                html.write(
                        "<br><a href=\"" + imgURL + "\">"
                        + html.img(imgURL, 150, 0, "right")
                        + "</a>\n");
            }
        } catch(Exception ex) {
            html.errorMsg(ex,  " DA 4");
        }    

        html.write("<table>\n");
        html.write(html.bar(html.element("h2", 
                html.superClassLink(service.getType(), link, "serviceclassuri", false),
                "servicetype", null)));

        try {
            advString = service.getLabel();
        } catch(Exception ex) {
            html.errorMsg(ex, " DA 3");
        }

        html.write(html.tr(html.td(html.element(
                "h2", advString, "advlabel", null), null, "colspan=\"2\"")));
        Iterator contacts = null; 
        try {
            contacts = service.getContacts();
        } catch(Exception ex) {
            html.errorMsg(ex, " DA 5");
        }

        String description = service.getDescription();
        html.write(html.pair("", description, "description"));

        while(contacts.hasNext()) {
            Contact con=(Contact)contacts.next();
            String http = con.getUrl();
            /* Both "www.x.dom" and "http://www.x.dom" occur.
             * Fix it here, use "www.x.dom".
             */
            if (http != null && http.startsWith("http://")) {
                http = http.substring(7, http.length());
            }
            html.write(html.pair("", con.getLabel(), "contactlabel"));
            html.write(html.pair("Opening hours", con.getOpeningHours()));
            html.write(html.pair("Address", con.getAddress()));
            html.write(html.pair("Telephone", con.getTelephone()));
            html.write(html.pair("Fax", con.getFax()));
            html.write(html.pairlink("Email", con.getEmail(), null, "mailto"));
            html.write(html.pairlink("", http, null, "http://"));
            if (contacts.hasNext()) {
                html.write(html.tr(html.td(html.hr(), null, "colspan=\"2\"")));
            }
        }
        html.write("</table>\n");
        html.write("\n<!-- DisplayAdvertisement done -->\n");
        return SKIP_BODY;
    }
}
