package fi.helsinki.cs.keltsi.tags;

import fi.helsinki.cs.keltsi.jenardf.Keltsi;
import fi.helsinki.cs.keltsi.jenardf.KeltsiClass;
import fi.helsinki.cs.keltsi.jenardf.Service;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.JspWriter;
import java.util.Iterator;

/**
 * This tag handler displays a search field 
 * by which user is able to input keywords and 
 * that way pick out a service class from the ontology.
 * 
 *  @author Arno Aalto
 */
public class DisplayServiceClass extends TagSupport {

    private static boolean displayInstances = false;    
    private Html html;
    private KeltsiClass serviceClass;
    private Link link;
    private String synonym;

    public int doStartTag() throws JspException {
        init();
        html.write("<form action=\"" + link.url(null, null, null) + "\""
                       + "method=\"post\">"
                   + "<table class=\"servicetable\" "
                           + "cellspacing=\"0\" "
                           + "cellpadding=\"2\" "
                           + "width=\"100%\">\n");

        displaySearchBar();
        displaySuperClasses();
        displayThisClass();
        displaySubClasses();
        if (displayInstances) {
            displayInstanceList();
        }
        html.write("</table>\n</form>\n");
        return SKIP_BODY;
    }

    private void init() {
        try {
            HttpServletRequest req =
                    (HttpServletRequest)pageContext.getRequest();
            html = new Html(pageContext.getOut());
            serviceClass = (KeltsiClass) pageContext
                    .getAttribute("fi.helsinki.cs.keltsi.ServiceClass");
            if (serviceClass == null) {
                throw new JspException("No serviceclass found!");
            }
            link  = (Link) pageContext
                    .getAttribute("fi.helsinki.cs.keltsi.ServiceClassLink");
            synonym = req.getParameter("serviceclasssynonym");
            if (synonym != null) {
                link.removeParam("serviceclasssynonym");
            } else {
                synonym = "";
            }
        } catch(Exception ex) {
            html.errorMsg(ex);
        }
    }

    private void displaySearchBar() {
        html.write(
            html.bar("Service shortcut:"
                      + "<input type=\"text\"/ "
                      +   "size=\"12\""
                      +   "value=\"" + synonym + "\" "
                      +   "name=\"serviceclasssynonym\" />"
                      + "<input type=\"submit\" "
                      +   "name=\"synonym\" value=\"go!\" />"
                      ));
    }

    private void displaySuperClasses() {
        Iterator sClasses = serviceClass.getSuperClasses(true);
        while (sClasses.hasNext()) {
            KeltsiClass sClass = (KeltsiClass)sClasses.next();
            html.write(
                html.tr(
                    html.td(
                        html.superClassLink(sClass, link, "serviceclassuri", 
                                            false))
                  + html.td("" + sClass.getInstanceCount(true), 
                            null, "align=\"right\"")));
        }
    }

    private void displayThisClass() {
        html.write(
            html.tr(
                html.td(html.thisClass(serviceClass))
              + html.td("" + serviceClass.getInstanceCount(true), 
                        null, "align=\"right\"")));
    }

    private void displaySubClasses() {
        Iterator sClasses = serviceClass.getSubClasses(false);
        while (sClasses.hasNext()) {
            KeltsiClass sClass = (KeltsiClass)sClasses.next();
            html.write(
                html.tr(
                    html.td(
                        html.subClassLink(sClass, link, "serviceclassuri"))
                  + html.td("" + sClass.getInstanceCount(true), 
                            null, "align=\"right\"")));
        }
    }

    private void displayInstanceList() {
        Iterator services = serviceClass.getInstances(false);
        while (services.hasNext()) {
            Service service = (Service)services.next();
            html.write(
                html.tr(
                    html.td(
                        html.serviceLink(service, link),
                                         null, "colspan=\"2\"")));
        }
    }
}
