package fi.helsinki.cs.keltsi.tags;

import fi.helsinki.cs.keltsi.jenardf.Keltsi;
import fi.helsinki.cs.keltsi.jenardf.KeltsiClass;
import fi.helsinki.cs.keltsi.jenardf.Service;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.JspWriter;
import java.util.Iterator;

/**
 *  Class to write html to JspWriter
 */
public class Html {
    // TODO: Change to relative links and include icons in distribution package
    public static final String serviceIcon =
            "http://www.cs.Helsinki.FI/group/keltsi/icons/service.gif";
    public static final String thisClassIcon =
            "http://www.cs.Helsinki.FI/group/keltsi/icons/thisclass.gif";
    public static final String subClassIcon = 
            "http://www.cs.Helsinki.FI/group/keltsi/icons/subclass.gif";
    public static final String superClassIcon = 
            "http://www.cs.Helsinki.FI/group/keltsi/icons/superclass.gif";

    protected JspWriter out;
    
    public Html(JspWriter out) {
        this.out = out;
    }

    public void errorMsg(String msg) {
        try {
            out.write("<strong>" + msg + "</strong>");
        } catch(Exception ex) {
        }
    }

    public void errorMsg(Exception ex) {
        try {
            PrintWriter pw = new PrintWriter(out, true);
            out.write("<xmp>");
            ex.printStackTrace(pw);
            out.write("</xmp>");
        } catch(Exception e) {
        }
    }
    
    public void errorMsg(Exception ex, String msg) {
        errorMsg(ex);
        errorMsg(msg);
        hr();
    }
    public void write(String string) {
        if (string != null) {
            try {
                out.write(string);
            } catch(Exception ex) {}
        }
    }

    public static String img(String url, int width, int height, String align) {
        String img =  "<img src=\"" + url + "\" ";
        if (height > 0) {
            img += "height=\"" + height + "\" ";
        }
        if (width > 0) {
            img += "width=\"" + width + "\" ";
        }
        if (align != null) {
            img += "align=\"" + align + "\" ";
        }
        img += "/>";
        return img;
    }
    
    public static String img(String url) {
        return img(url, 0, 0, null);
    }
    
    public static String hr() {
        return "<hr />\n";
    }

    public static String br() {
        return "<br />\n";
    }
    
    public static String tr(String text, String cssClass, String args) {
        return "\t" + element("tr", "\n\t\t" + text + "\t", cssClass, args) + "\n";
    }

    public static String tr(String text, String cssClass) {
        return tr(text, cssClass, null);
    }

    public static String tr(String text) {
        return tr(text, null, null);
    }

    public static String td(String text, String cssClass, String args) {
        return "\t\t" + element("td", text, cssClass, args) + "\n";
    }

    public static String td(String text, String cssClass) {
        return td(text, cssClass, null);
    }

    public static String td(String text) {
        return td(text, null, null);
    }
    
    public static String span(String text, String cssClass) {
        return element("span", text, cssClass, null);
    }

    public static String div(String text, String cssClass) {
        return element("div", text, cssClass, null);
    }

    public static String trimLabel(String labelString) {
        if (labelString == null) {
            return null;
        } else if (labelString.length() > 28) {
            return labelString.substring(0,27) + ".." ;
        } else {
            return labelString;
        }
    }   

    public static String element(String tag, String value, String cssClass, String args) {
        if (value == null || tag == null) {
            return null;
        }
        if (args == null || args.length() == 0) {
            args = "";
        } else {
            args = " " + args;
        }
        if (cssClass != null) {
            args  += " class=\"" + cssClass + "\"";
        }
        return "<" + tag + args + ">" + value +"</" + tag + ">";
    }

    public static String pair(String name, String value, String cssClass) {
        if (value == null) {
            return null;
        }
        String nameClass = null;
        String valueClass = null;
        if (cssClass != null) {
            nameClass  = cssClass + "Name";
            valueClass = cssClass + "Value";
        }
        return tr(  td(name, nameClass)
                  + td(value, valueClass));
    }

    public static String pair(String name, String value) {
        return pair(name, value, "");
    }
    
    public static String pairlink(String name, String value, 
                         String cssClass, String protocol) {
        if (value == null) {
            return null;
        }
        value = "<a href=\"" + protocol + value + "\">" + value + "</a>";
        return pair(name, value, cssClass);
    }
    
    private static String anyLink(String icon, String link, String cssClass) {
        return span(img(icon) + " " + link, cssClass);
    }
    
    public static String superClassLink(KeltsiClass kClass, Link link, 
                                        String name, boolean trim) {
        String label = kClass.getLabel();
        String label2 = trim ? trimLabel(label) : label;
        return anyLink(superClassIcon, 
                       link.aHref("ontology", "search.jsp", name, kClass.getURI(),
                                  label2, "Move to upper class: " + label),
                        "superclasslink");
    }
    public static String superClassLink(KeltsiClass kClass, Link link, 
                                        String name) {
        return superClassLink(kClass, link, name, true);
    }

    public static String subClassLink(KeltsiClass kClass, Link link, 
                                        String name) {
        String label = kClass.getLabel();
        return anyLink(subClassIcon,
		       link.aHref("ontology", "search.jsp", name, kClass.getURI(),
                                  trimLabel(label), "Move to subclass: " + label),
                        "subclasslink");
    }

    public static String thisClass(KeltsiClass kClass) {
        return span(img(thisClassIcon) + " " + kClass.getLabel(), "thisclass");
    }

    public static String serviceLink(Service service, Link link) {
        String label = service.getLabel();
        String parent = "";
        KeltsiClass sClass = service.getType();
        if(sClass != null) {
            String parentLabel = sClass.getLabel();
            parent = link.aHref("ontology", "search.jsp", "serviceclassuri",
                                sClass.getURI(), trimLabel(parentLabel),
                                "Open class: " + parentLabel) + " / ";
        }
        return img(serviceIcon) + " "
               + parent 
               + link.aHref("detail", "detail.jsp", "serviceuri", 
                            service.getURI(), label, 
                            "View service: " + label);
    }

    public static String bar(String text) {
        return tr(td(text, null, "bgcolor=\"#d0d0a6\" colspan=\"2\""));
    }

}
