package fi.helsinki.cs.keltsi.tags;

import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Enumeration;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;

/**
 *  Class for holding and manipulating the name-value-pairs of the query
 *  and to construct html a-elements.
 * 
 *  @author Arno Aalto
 */
public class Link {
    private HashMap param;
    private String submit;
    private String page;

    public Link(HttpServletRequest request) {
        page = request.getRequestURI();
        param = new HashMap();
        Enumeration names = request.getParameterNames();
        while(names.hasMoreElements()) {
            // Only one value per name
            String name = (String) names.nextElement();
            // Special cases
            if ("submit".equals(name)) {
                submit = request.getParameter(name);
            } else {
                param.put(name, request.getParameter(name));
            }
        }
    }
    
    public String getPage() {
        return page;
    }

    public HashMap getMap() {
        return param;
    }

    public boolean isSubmit() {
        return (submit != null);
    }

    public String getSubmit() {
        return submit;
    }

    public String addParamStr(String paramStr, String name, String value) {
        if (paramStr.length() > 0) {
            paramStr += "&";
        }
        paramStr += URLEncoder.encode(name)  + "=" + URLEncoder.encode(value);
        return paramStr;
    }

    private String params(String name, String value) {
        String result = "";
        Iterator names = param.keySet().iterator();
        while(names.hasNext()) {
            String aName = (String) names.next();
            if (aName.equals(name)) {
                if (value != null) {
                    result = addParamStr(result, aName, value);
                }
            } else {
                result = addParamStr(result, aName, (String) param.get(aName));
            }
        }
        if (name != null && value != null && !param.containsKey(name)) {
            result = addParamStr(result, name, value);
        }
        return result;
    }

    public void removeParam(String name) {
        param.remove(name);
    }

    public void addParam(String name, String value) {
        param.put(name, value);
    }

    public String url(String page, String name, String value) {
        // page defaults to current page
        if (page == null) {
            page = getPage();
        }
        return page + "?" + params(name, value);
    }

    public String aHref(String target,
                        String page,
                        String name,
                        String value,
                        String label,
                        String title) {
        String result = "<a";
        if (target != null) {
            result += " target=\"" + target + "\" ";
        }
        result += " href=\"" + url(page, name, value) + "\"";
        if (title != null) {
            result += " title=\"" + title;
        }
        result += "\">" + label + "</a>";
        return result;
    }
    
    public String aHref(String target,
                        String page,
                        String name,
                        String value,
                        String label,
                        String title,
                        String value2) {
        String result = "<a";
        if (target != null) {
            result += " target=\"" + target + "\" ";
        }
            result += " href=\"" + url(page, name, value);
        if (target == "_top") {
            result = addParamStr(result, "serviceclassuri", value2);
        }                                                  
        result += "\"";
        
        if (title != null) {
            result += " title=\"" + title;
        }
        result += "\">" + label + "</a>";
        return result;
    }
}
