package fi.helsinki.cs.keltsi.tags;

import fi.helsinki.cs.keltsi.jenardf.Keltsi;
import fi.helsinki.cs.keltsi.jenardf.KeltsiDAML;
import fi.helsinki.cs.keltsi.jenardf.Ontology;
import fi.helsinki.cs.keltsi.jenardf.KeltsiClass;

import com.hp.hpl.jena.daml.DAMLClass;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.JspWriter;
import java.util.Iterator;

/**
 * This tag handler controls and displays browsing
 * of ontologies.
 * 
 *  @author Arno Aalto
 */
public class OntologySearch extends TagSupport {
    protected Link link;
    protected KeltsiClass ontoClass;
    protected Html html;
    protected String[] verbs;
    protected String verb;
    protected KeltsiDAML model;
    protected KeltsiClass serviceClass;
    HttpServletRequest request;
    
    public int doStartTag() throws JspException {
        init();
        html.write("<table class=\"servicetable\" "
                        + "cellspacing=\"0\" "
                        + "cellpadding=\"2\" "
                        + "width=\"100%\">\n");
        Iterator ontologies = model.getOntologies();
        while (ontologies.hasNext()) {
            Ontology ontology = (Ontology) ontologies.next();
            KeltsiClass ontoTop = ontology.getDomainTop();
            if(serviceClass.equals(ontoTop) 
                    || serviceClass.hasSuperClass(ontoTop)) {
                displayOntology(ontology);
            } else {
                link.removeParam(ontology.getLabel());
                link.removeParam(ontology.getLabel() + "verb");
            }
        }
        html.write("</table>\n");
        return SKIP_BODY;
    }

    private void displayOntology(Ontology ontology) {
        String[] verbs = null;
        verb = null;
        boolean hasOntoClass = false;
        String paramName = null;
        KeltsiClass ontoClass = null;
        KeltsiClass ontoRoot;
        try {
            /* Get value of parameter named for this ontology */
            paramName = ontology.getLabel();
            String ontoClassUri = request.getParameter(paramName);

            /* Try to get corresponding KeltsiClass object */
            if (ontoClassUri  != null) {
                ontoClass = (KeltsiClass) model.getKeltsiResource(ontoClassUri);
            }
            ontoRoot = ontology.getRangeTop();
            verbs = ontology.getVerbs();
    
            hasOntoClass = (ontoClass != null);
            if (!hasOntoClass) {
                ontoClass = ontoRoot;
            }
            verb = request.getParameter(paramName + "verb");
            if ((verb == null || verb.length() == 0)
                    && verbs != null 
                    && verbs.length > 0) {
                // First verb is default
                verb = verbs[0];
                link.addParam(paramName + "verb", verb);
            }
        } catch(Exception ex) {
            html.errorMsg(ex);
            return;
        }

        if (hasOntoClass) {
            // write verbs as a list
            verbs = ontology.getVerbs();
            String verbList = "";
            if (verbs != null) {
                int verbCount = verbs.length;
                int index = 0;
                while(index < verbCount) {
                    String v = verbs[index];
                    verbList += verbLink(verbs[index], paramName + "verb");
                    ++index;
                }
            }            
            html.write(
                html.bar(link.aHref(null, null, paramName, null, "x",
                                   "Close this ontology")
                         + " " + verbList));
            Iterator sClasses = ontoClass.getSuperClasses(true);
            KeltsiClass sClass;
            while (sClasses.hasNext()) {
                sClass = (KeltsiClass) sClasses.next();
                html.write(
                    html.tr(
                        html.td(
                            html.superClassLink(sClass, link, paramName))));
            }

            html.write(html.tr(html.td(html.thisClass(ontoClass))));

            sClasses = ontoClass.getSubClasses(false);
            while (sClasses.hasNext()) {
                sClass = (KeltsiClass) sClasses.next();
                html.write(
                    html.tr(
                        html.td(
                            html.subClassLink(sClass, link, paramName))));
            }
        } else {
            String label = ontoClass.getLabel();
            html.write(
                html.bar(link.aHref(null, null, paramName, ontoRoot.getURI(),
                                   label, "Open " + label)));
        }
    }

    private String verbLink(String value, String paramVerb)  {
        if (value.equals(verb)) {
            return "<b>" + value + "</b> ";
        } else {
            return link.aHref(null, null, paramVerb, value, value, null) 
                   + " ";
        }
    }


    private void init() {
        try {
            request =
                    (HttpServletRequest)pageContext.getRequest();
            html = new Html(pageContext.getOut());
            link  = (Link) pageContext
                    .getAttribute("fi.helsinki.cs.keltsi.ServiceClassLink");

            model = (KeltsiDAML) pageContext
                    .findAttribute("fi.helsinki.cs.keltsi.KeltsiDAML");

            serviceClass = (KeltsiClass) pageContext
                    .findAttribute("fi.helsinki.cs.keltsi.ServiceClass");
            if(serviceClass == null) {
                throw new Exception("No serviceclass found!");
            }
        } catch(Exception ex) {html.errorMsg(ex);}
    }
}
