package fi.helsinki.cs.keltsi.tags;

import fi.helsinki.cs.keltsi.util.*;
import fi.helsinki.cs.keltsi.jenardf.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.jena.vocabulary.DAML_OIL;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDFS;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.JspException;
import java.util.*;
import javax.servlet.jsp.PageContext;

/**
 * This tag handler displays links to other services
 * associated with the advertisement of the assigned 
 * service.
 */
public class OtherServices extends TagSupport {

    private Service service;
    private Link link;
    private KeltsiDAML model;
    private Html html;
    private Advertisement adv;
    
    private void errorMsg(String msg) {
        try {
            pageContext.getOut().write("<pre>" + msg + "</pre>");
        } catch(Exception ex) {}
    }
    
    public int doStartTag() throws JspException {
        String aHref2instance = null;
        String instanceURI = "";
        
        html = new Html(pageContext.getOut());
        html.write("<!-- OtherServices starting -->\n");

        /* Get Instance URI as a parameter. */
        try {
            HttpServletRequest req =
              (HttpServletRequest)pageContext.getRequest();
            
            link = new Link(req);
            
            instanceURI = req.getParameter("serviceuri");
        } catch (Exception ex) {
            html.errorMsg(ex, "OtherServices initializing");
        }

        /* Get the model from scope. */
        try {
            model = (KeltsiDAML) pageContext.findAttribute(
                    "fi.helsinki.cs.keltsi.KeltsiDAML");
        } catch (Exception ex) {
            html.errorMsg(ex, "getting model from scope");
        }

        /* Get service instance. */
        service = (Service) model.getKeltsiResource(instanceURI);
        if (service == null) {
            html.errorMsg("Instance " + instanceURI + " was not found.");
        } else {
            try {
                adv = service.getAdvertisement();
                if (adv == null) {
                    throw new Exception("No advertisement found!");
                }
            } catch(Exception ex) {
                html.errorMsg(ex);
            }
            displayServices(adv.getServices(), "Other services by");
        }
        displayServices(service.getServiceValues(Keltsi.provides),
                    "Also provided by");
        displayServices(service.getServiceValues(Keltsi.additionalServices),
                    "Also available at");
        displayServices(service.getServiceValues(Keltsi.recommends),
                    "Recommended by");
        html.write("\n<!-- OtherServices done -->\n");
        return SKIP_BODY;
    }

    private void displayServices(Iterator iter, String label) {
        Service serviceInstance = null;
        boolean first = true;
        while (iter.hasNext()) {
            serviceInstance = (Service) iter.next();
            if (!service.equals(serviceInstance)) {
                if (first) {
                    html.write(
                            html.hr()
                            + label + " <b>"
                            + adv.getLabel() + ":</b>"
                            + html.br());
                    first = false;
                }
                html.write(html.serviceLink(serviceInstance, link) + html.br());
            }
        }
    }

}
