package fi.helsinki.cs.keltsi.tags;

import fi.helsinki.cs.keltsi.util.*;
import fi.helsinki.cs.keltsi.jenardf.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.common.*;

import java.io.*;
import java.util.Iterator;
import java.util.HashMap;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.JspWriter;

/** 
 *  This tag handler displays search-link, count of results or 
 *  resultlist.
 * 
 *  @author Arno Aalto
 */
public class Search extends TagSupport {
    protected Link link;
    protected String page;
    protected Html html;
    protected KeltsiDAML model;

    public int doStartTag() throws JspException {
        init();
        html.write("<table class=\"servicetable\" "
                        + "cellspacing=\"0\" "
                        + "cellpadding=\"2\" "
                        + "width=\"100%\">\n");
        if (link.isSubmit()) {
            resultList();
        } else {
            html.write(html.tr(html.td(
                    submitControl()
                    + " (" + resultCount() + ")")));
        }
        /* Hide debug-info into comment */
        html.write("\n<!-- " + model.getPreviousMapQueryString() + " -->\n");
        html.write("</table>\n");
        return SKIP_BODY;
    }
    
    protected String submitControl() {
        return "<hr width=\"100%\" /><font size='4' color='#67ff00' face='arial'>"
               + link.aHref(null, page, "submit", "do", "LIST RESULTS", 
                           "List the matching services")
               + "</font>";
    }
    
    protected void resultList() {
        Service service = null;
        String list = "";
        Iterator results = model.mapQuery(link.getMap(), true);
        if (results == null) {
            // TODO: Display something?
        } else {
            int count = 0;
            while(results.hasNext()) {
                ++ count;
                service = (Service) model.getKeltsiResource(
                        (DAMLInstance) results.next());
                list += html.tr(html.td(
                        html.serviceLink(service, link)));
            }
            html.write(list);
        }
    }

    protected String resultCount() {
        Iterator results = model.mapQuery(link.getMap(), true);
        if (results == null) {
            return "0";
        } else {
            int count = 0;
            while(results.hasNext()) {
                ++count;
                results.next();
            }
            return new Integer(count).toString();
        }
    }

    protected void init() {
        try {
            HttpServletRequest req =
                    (HttpServletRequest)pageContext.getRequest();
            page = req.getRequestURI();
            html = new Html(pageContext.getOut());
            link  = (Link) pageContext
                    .getAttribute("fi.helsinki.cs.keltsi.ServiceClassLink");
            model = (KeltsiDAML) pageContext
                    .findAttribute("fi.helsinki.cs.keltsi.KeltsiDAML");
        } catch(Exception ex) {
            html.errorMsg(ex);
        }
    }
}
