package fi.helsinki.cs.keltsi.tags;

import fi.helsinki.cs.keltsi.util.*;
import fi.helsinki.cs.keltsi.jenardf.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.jena.vocabulary.DAML_OIL;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDFS;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.JspException;
import java.util.*;
import javax.servlet.jsp.PageContext;

/**
 * This tag handler displays instances and classes, that are 
 * somehow meaningfully associated with a assigned instance 
 * of a service. One way association is done in ontology.
 * So in practise, this tag handler finds associations by 
 * searching the ontology for values of the specified properties.
 * Inverse properties and their values are also searched and
 * displayd.
 */
public class SemanticLinks extends TagSupport {

    private KeltsiDAML model;
    private Link link;
    private Html html;
    private Service service = null;
    private String serviceClassURI = "";
    private static final String[][] propURIs = {
        {"service",         "http://protege.stanford.edu/Keltsi#franchiseeOf"},
        {"serviceinverse",  "http://protege.stanford.edu/Keltsi#franchisee"},
        {"productclass",    "http://protege.stanford.edu/Keltsi#rents"},
        {"productclass",    "http://protege.stanford.edu/Keltsi#retails"},
        {"serviceclass",    "http://protege.stanford.edu/Keltsi#franchises"},
        {"location",        "http://protege.stanford.edu/Keltsi#location"},
        {"restauranttype",  "http://protege.stanford.edu/Keltsi#restaurantType"},
        {"ambiance",        "http://protege.stanford.edu/Keltsi#ambiance"},
        {"rating",          "http://protege.stanford.edu/Keltsi#rating"}};

    public DAMLProperty getKeltsiProperty(String uri) {
        return (DAMLProperty) model.getDAMLValue(uri);
    }

    public int doStartTag() throws JspException {
        DAMLProperty prop = null;

        String serviceURI = "";
        html = new Html(pageContext.getOut());
        html.write("<!-- SemanticLinks starting -->\n");

        /* Get Instance URI as a parameter. */
        try {
            HttpServletRequest req =
              (HttpServletRequest)pageContext.getRequest();

            link = new Link(req);

            serviceURI = req.getParameter("serviceuri");

        } catch (Exception ex) {
            html.errorMsg(ex, "while getting request params.");
        }

        /* Get the model from scope. */
        try {
            model = (KeltsiDAML) pageContext.findAttribute(
                    "fi.helsinki.cs.keltsi.KeltsiDAML");
            if (model == null) {
                throw new Exception("Model is null!");
            }
        } catch (Exception ex) {
            html.errorMsg(ex, "while getting the model from scope");
        }

        /* Get service instance. */
        service = (Service) model.getKeltsiResource(serviceURI);
        if (service == null) {
            html.errorMsg("Instance " + serviceURI + " was not found.");
        }
        else {
            /* Get Keltsi properties. */
            html.write("<table>\n");
            for(int i=0; i<propURIs.length; ++i) {
                if (propURIs[i][0] != "serviceinverse") {
                    prop = getKeltsiProperty(propURIs[i][1]);
                } else {
                    prop = model.getInverseProperty(propURIs[i][1]);
                }
                if (prop == null) {
                    html.errorMsg("Property " + propURIs[i][1] + " was not found.");
                } else {
                    /* Print Keltsi properties and their values. */
                    writeValues(prop, propURIs[i][0]);
                }
            }
            html.write("</table>\n");
        }
        html.write("\n<!-- SemanticLinks done -->\n");
        return SKIP_BODY;
    }

    private void writeValues(DAMLProperty prop, String propertyType) {
        Iterator valuesIter = null;
        Service serviceInstance = null;
        String list = "";
        KeltsiClass kClass = null;

        /* Get values of property. */
        if (propertyType == "service") {
            valuesIter = service.getServiceValues(prop);
        } else if(propertyType == "serviceinverse") {
            valuesIter = service.getServiceInverseValues(prop);
        } else {
            valuesIter = service.getClassValues(prop);
        }
        if (valuesIter != null) {
            String propertyName = prop.getLocalName();
            if (propertyType == "serviceinverse") {
                propertyName = model.inverseName(propertyName);
            }
            int rows = 0;
            while(valuesIter.hasNext()) {
                if (propertyType == "service" || propertyType == "serviceinverse") {
                    serviceInstance = (Service) valuesIter.next();
                    list += Html.td(Html.serviceLink(serviceInstance, link));
                } else if(propertyType == "serviceclass") {
                    kClass = (KeltsiClass) valuesIter.next();
                    list += Html.td(Html.superClassLink(kClass, link, "serviceclassuri"));
                } else {
                    kClass = (KeltsiClass) valuesIter.next();
                    list += Html.td(Html.superClassLink(kClass, link, propertyType));
                }
                rows++;
            }
            if (rows > 0) {
                html.write(html.tr(html.td(propertyName, null, 
                                           "rowspan=\"" + rows + "\"")
                                   + list));
            }
        }
    }
}    
