package fi.helsinki.cs.keltsi.jenardf;

import com.hp.hpl.jena.daml.DAMLModel;
import com.hp.hpl.jena.daml.DAMLClass;
import com.hp.hpl.jena.daml.DAMLProperty;
import com.hp.hpl.mesa.rdf.jena.model.RDFException;

/**
 *  Defines the vocabulary for the Keltsi-project.
 *  Static instances of resources and properies are provided.
 */
public class Keltsi{

    protected static final String uri =
            "http://protege.stanford.edu/Keltsi#";

    /** returns the URI for this schema
     * @return the URI for this schema
     */
    public static String getURI()
    {
        return uri;
    }

    public static       KeltsiClass ServiceRoot;
    public static       KeltsiClass ProductRoot;

           static final String   sService = "Service";
    public static       DAMLClass Service = null;
           static final String   sProduct = "Product";
    public static       DAMLClass Product = null;
           static final String   sAdvertisement = "Advertisement";
    public static       DAMLClass Advertisement = null;
           static final String   sContact = "Contact";
    public static       DAMLClass Contact = null;
           static final String   sImageFile = "ImageFile";
    public static       DAMLClass ImageFile = null;

    /* Properties of Service classes */
           static final String   ssynonym = "synonym";
    public static       DAMLProperty synonym;

    /* Properties of Service instances */
           static final String scontact = "contact";
    public static       DAMLProperty contact;

    /* Superpoperties of Service instances */
           static final String sproductClassProperty = "productClassProperty";
    public static       DAMLProperty productClassProperty;
           static final String sserviceClassProperty = "serviceClassProperty";
    public static       DAMLProperty serviceClassProperty;
           static final String sserviceProperty = "serviceProperty";
    public static       DAMLProperty serviceProperty;
           static final String sliteralProperty = "literalProperty";
    public static       DAMLProperty literalProperty;

    /* Properties of Contact */
           static final String saddress = "address";
    public static       DAMLProperty address;
           static final String stelephone = "telephone";
    public static       DAMLProperty telephone;
           static final String semail = "email";
    public static       DAMLProperty email;
           static final String sfax = "fax";
    public static       DAMLProperty fax;
           static final String surl = "url";
    public static       DAMLProperty url;
           static final String simageFile = "imageFile";
    public static       DAMLProperty imageFile;

    /* Properties of ImageFile */
           static final String   spath = "path";
    public static       DAMLProperty path;

    /* Properties of Advertisement */
           static final String   sservices = "service";
    public static       DAMLProperty services;
           static final String   slabel = "label";
    public static       DAMLProperty label;
    /* old stuff, removed
           static final String   sproviderOfService = "providerOfService";
    public static       DAMLProperty providerOfService;
    */    
           static final String   sdescription = "description";
    public static       DAMLProperty description;
           static final String   sopeningHours = "openingHours";
    public static       DAMLProperty openingHours;
           static final String   srecommends = "recommends";
    public static       DAMLProperty recommends;
           static final String   sprovides = "provides";
    public static       DAMLProperty provides;
           static final String   sadditionalServices = "additionalServices";
    public static       DAMLProperty additionalServices;

    public static void initialize(KeltsiDAML model){
        try {
        Service              = (DAMLClass)model.getDAMLValue(uri
                                    + sService);
        Product              = (DAMLClass)model.getDAMLValue(uri
                                    + sProduct);
        Advertisement        = (DAMLClass)model.getDAMLValue(uri
                                    + sAdvertisement);
        Contact              = (DAMLClass)model.getDAMLValue(uri
                                    + sContact);
        ImageFile            = (DAMLClass)model.getDAMLValue(uri
                                    + sImageFile);
        synonym              = (DAMLProperty)model.getDAMLValue(uri
                                    + ssynonym);
        contact              = (DAMLProperty)model.getDAMLValue(uri
                                    + scontact);
        productClassProperty = (DAMLProperty)model.getDAMLValue(uri
                                    + sproductClassProperty);
        serviceClassProperty = (DAMLProperty)model.getDAMLValue(uri
                                    + sserviceClassProperty);
        serviceProperty      = (DAMLProperty)model.getDAMLValue(uri
                                    + sserviceProperty);
        literalProperty      = (DAMLProperty)model.getDAMLValue(uri
                                    + sliteralProperty);
        address              = (DAMLProperty)model.getDAMLValue(uri
                                    + saddress);
        telephone            = (DAMLProperty)model.getDAMLValue(uri
                                    + stelephone);
        email                = (DAMLProperty)model.getDAMLValue(uri
                                    + semail);
        fax                  = (DAMLProperty)model.getDAMLValue(uri
                                    + sfax);
        url                  = (DAMLProperty)model.getDAMLValue(uri
                                    + surl);
        imageFile            = (DAMLProperty)model.getDAMLValue(uri
                                    + simageFile);
        path                 = (DAMLProperty)model.getDAMLValue(uri
                                    + spath);
        services             = (DAMLProperty)model.getDAMLValue(uri
                                    + sservices);
        label                = (DAMLProperty)model.getDAMLValue(uri
                                    + slabel);
        openingHours         = (DAMLProperty)model.getDAMLValue(uri
                                    + sopeningHours);
        recommends           = (DAMLProperty)model.getDAMLValue(uri
                                    + srecommends);
        provides             = (DAMLProperty)model.getDAMLValue(uri
                                    + sprovides);
        additionalServices   = (DAMLProperty)model.getDAMLValue(uri
                                    + sadditionalServices);
/*
        providerOfService    = (DAMLProperty)model.getDAMLValue(uri
                                    + sproviderOfService);
*/
        description          = (DAMLProperty)model.getDAMLValue(uri
                                    + sdescription);

        ServiceRoot = (KeltsiClass)model.getKeltsiResource(Service);
        ProductRoot = (KeltsiClass) model.getKeltsiResource(Product);
        }catch(Exception ex) {
//            model.error = ex.toString();
        }
    }
}
