package fi.helsinki.cs.keltsi.jenardf;

import com.hp.hpl.jena.daml.*;
import java.util.Hashtable;
import java.util.LinkedList;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDFS;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;

public abstract class KeltsiResource extends Object {
    protected DAMLCommon resource;

    KeltsiResource(DAMLCommon resource) {
        this.resource = resource;
    }
    
    protected KeltsiDAML getModel() {
        return (KeltsiDAML) resource.getModel();
    }

    public KeltsiClass getType() {
        try {
            DAMLClass dClass = (DAMLClass) resource.getPropertyValue(RDF.type);
            return (KeltsiClass) getModel().getKeltsiResource(dClass);
        } catch(Exception ex) {
            return null;
        }
    }

    public DAMLCommon getResource() {
        return resource;
    }

    public String getURI() {
        return resource.getURI();
    }

    public String getNameSpace() {
        return resource.getNameSpace();
    }

    public String getLabel() {
        Statement s = null;
        /* Keltsi.label, workaround
            Protege doesn't allow to write meaningful labels for instances
        */
        try {
            s = resource.getProperty(Keltsi.label);
            if(s != null) {
                return s.getString();
            }
        } catch(Exception ex) {}
        try {
            s = resource.getProperty(RDFS.label);
            if(s != null) {
                return s.getString();
            }
        } catch(Exception ex) {}
        try {
            return resource.getLocalName();
        } catch(Exception ex) {}
        return resource.toString();
    }

    /* 
     *  Is this object equal to this one.
     *  The decision is droped to the jena-layer,
     *  which makes it by comparing the URIs of
     *  the Resources.
     *  @param object The object to compare this one to.
     *  @return True if the object equals this one.
     */
    public boolean equals(Object object) {
        if (object instanceof KeltsiResource) {
            return getResource()
                    .equals(((KeltsiResource) object).getResource());
        } else {
            return false;
        }
    }

    /*
     * Return stringvalue of property. The range of the property is
     * expected to be rdf:Literal.
     */
    public String getSingleLiteralValue(Property property) {
        RDFNode node = null;
        try {
            node = resource.getProperty(property);
        }
        catch (Exception ex) {
            return null;
        }
        if (node == null) return 
                "<null value (shouldn't happen according to Jena docs)>";
        if (!(node instanceof Statement)) return 
                "<not a statement (shouldn't happen according to Jena docs)>";
        try {
            return ((Statement) node).getString();
        }
        catch (Exception ex) {
            return "<not a literal>";
        }
    }
}

