/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Ian Dickinson, HP Labs Bristol
 * Author email       Ian_Dickinson@hp.com
 * Package
 * Created            4 Jan 2002
 * Filename           $RCSfile: ClassInspector.java,v $
 * Revision           $Revision: 1.1 $
 * Release status     Experimental. Do not distribute. $State: Exp $
 *
 * Last modified on   $Date: 2002/01/10 14:18:50 $
 *               by   $Author: ijd $
 *
 * (c) Copyright Hewlett-Packard Company 2002
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

// Package
///////////////
package examples.daml.ex1;


// Imports
///////////////
import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.util.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import java.awt.event.*;



/**
 * Value inspector for DAML class expressions
 *
 * @author Ian Dickinson, HP Labs (<a href="mailto:Ian_Dickinson@hp.com">email</a>)
 * @version CVS info: $Id: ClassInspector.java,v 1.1 2002/01/10 14:18:50 ijd Exp $
 */
public class ClassInspector
    extends JFrame
    implements DAMLInspector
{

    // Constants
    //////////////////////////////////


    // Static variables
    //////////////////////////////////


    // Instance variables
    //////////////////////////////////

    /** The DAML class this inspector is showing */
    private DAMLClass m_class = null;

    /** The application object */
    private DAMLViewer m_viewer = null;

    /** My menu entry on the window menu */
    private JMenuItem m_mnuItem = null;


    // list models
    private DAMLListModel m_propertiesModel = null;
    private DAMLListModel m_equivsModel = null;
    private DAMLListModel m_supersModel = null;
    private DAMLListModel m_subsModel = null;
    private DAMLListModel m_instancesModel = null;

    /* JBuilder generated instance variables for GUI layout */
    BorderLayout borderLayout1 = new BorderLayout();
    JPanel pnlDescription = new JPanel();
    GridLayout gridLayout1 = new GridLayout();
    JPanel pnlValues = new JPanel();
    GridLayout gridLayout2 = new GridLayout();
    JPanel pnlPropertyGroup = new JPanel();
    JPanel pnlInstances = new JPanel();
    JPanel pnlRelations = new JPanel();
    JPanel pnlEquivs = new JPanel();
    JPanel pnlSupers = new JPanel();
    JPanel pnlSubs = new JPanel();
    GridLayout gridLayout3 = new GridLayout();
    GridLayout gridLayout4 = new GridLayout();
    Border border1;
    TitledBorder titledBorder1;
    Border border2;
    GridLayout gridLayout5 = new GridLayout();
    GridLayout gridLayout6 = new GridLayout();
    JScrollPane jScrollPane2 = new JScrollPane();
    JList lstEquivs = new JList();
    JPanel pnlProperties = new JPanel();
    BorderLayout borderLayout3 = new BorderLayout();
    JScrollPane jScrollPane1 = new JScrollPane();
    JList lstProperties = new JList();
    JScrollPane jScrollPane3 = new JScrollPane();
    JList lstSuperClasses = new JList();
    JScrollPane jScrollPane4 = new JScrollPane();
    JList lstSubClasses = new JList();
    GridLayout gridLayout7 = new GridLayout();
    GridLayout gridLayout8 = new GridLayout();
    JScrollPane jScrollPane5 = new JScrollPane();
    JList lstInstances = new JList();
    JScrollPane jScrollPane6 = new JScrollPane();
    JTextArea txtDescription = new JTextArea();
    Border border3;
    TitledBorder titledBorder2;
    Border border4;


    // Constructors
    //////////////////////////////////

    /**
     * Construct a class inspector for the given DAMLClass
     */
    public ClassInspector( DAMLClass cls, DAMLViewer viewer ) {
        m_class = cls;
        m_viewer = viewer;

        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }


    // External signature methods
    //////////////////////////////////

    /**
     * Make the inspector visible for the first time
     * @see makeVisible
     */
    public void showInspector() {
        // initialise the inspector's data
        setTitle( "Class Inspector for " + m_viewer.renderDAMLObjectName( m_class ) );
        try {
            String desc = m_class.prop_comment().getValue().getString();
            if (desc.length() > 0) {
                txtDescription.setText( desc );
            }
        }
        catch (RDFException ignore) {}
        catch (NullPointerException ignore) {}

        // set up the list models
        m_equivsModel = new DAMLListModel( m_viewer, (DAMLModel) m_class.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_class.getEquivalenceSet();
                                                }
                                            },
                                            true );
        m_subsModel = new DAMLListModel( m_viewer, (DAMLModel) m_class.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_class.getSubClasses();
                                                }
                                            },
                                            true );
        m_supersModel = new DAMLListModel( m_viewer, (DAMLModel) m_class.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_class.getSuperClasses();
                                                }
                                            },
                                            true );
        m_instancesModel = new DAMLListModel( m_viewer, (DAMLModel) m_class.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_class.getInstances();
                                                }
                                            },
                                            true );
        m_propertiesModel = new DAMLListModel( m_viewer, (DAMLModel) m_class.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_class.getDefinedProperties();
                                                }
                                            },
                                            true );

        lstEquivs.setModel( m_equivsModel );
        lstSubClasses.setModel( m_subsModel );
        lstSuperClasses.setModel( m_supersModel );
        lstInstances.setModel( m_instancesModel );
        lstProperties.setModel( m_propertiesModel );

        // make myself visible
        setSize( 500, 350 );
        setVisible( true );
    }


    /**
     * Answer the title of the inspector
     */
    public String getMenuTitle() {
        return "Class " + m_viewer.renderDAMLObjectName( m_class );
    }


    /**
     * Ensure that the inspector visible (even if iconified) and has focus
     */
    public void reveal() {
        setState( Frame.NORMAL );
        show();
        toFront();
        requestFocus();
    }


    /**
     * Set a reference to my entry in the window menu
     *
     */
    public void setWindowMenuEntry( JMenuItem mnu ) {
        m_mnuItem = mnu;
    }


    public void dispose() {
        m_viewer.getFrame().removeWindowMenuEntry( m_mnuItem );
        super.dispose();
    }


    // Internal implementation methods
    //////////////////////////////////

    /**
     * GUI layout - generated by JBuilder's design tool
     */
    private void jbInit() throws Exception {
        ListCellRenderer damlRenderer = new DefaultListCellRenderer() {
            public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
                // override the toString call on the DAML value
                return super.getListCellRendererComponent( list,
                                                           m_viewer.renderDAMLObjectName( value ),
                                                           index, isSel, hasFocus );
            }
        };
        ListCellRenderer superClassListRenderer = new DefaultListCellRenderer() {
            public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
                // override the toString call on the DAML value
                Component c = super.getListCellRendererComponent( list,
                                                                  m_viewer.renderDAMLObjectName( value ),
                                                                  index, isSel, hasFocus );

                // bold immediate super-classes
                for (Iterator i = m_class.getSuperClasses( false );  i.hasNext(); ) {
                    if (i.next().equals( value )) {
                        // an immediate parent
                        if (c instanceof JLabel) {
                            Font f = ((JLabel) c).getFont();
                            ((JLabel) c).setFont( new Font( f.getName(), Font.BOLD, f.getSize() ) );
                            break;
                        }
                    }
                }

                return c;
            }
        };
        ListCellRenderer subClassListRenderer = new DefaultListCellRenderer() {
            public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
                // override the toString call on the DAML value
                Component c = super.getListCellRendererComponent( list,
                                                                  m_viewer.renderDAMLObjectName( value ),
                                                                  index, isSel, hasFocus );

                // bold immediate super-classes
                for (Iterator i = m_class.getSubClasses( false );  i.hasNext(); ) {
                    if (i.next().equals( value )) {
                        // an immediate parent
                        if (c instanceof JLabel) {
                            Font f = ((JLabel) c).getFont();
                            ((JLabel) c).setFont( new Font( f.getName(), Font.BOLD, f.getSize() ) );
                            break;
                        }
                    }
                }

                return c;
            }
        };
        ListCellRenderer propertiesListRenderer = new DefaultListCellRenderer() {
            public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
                // override the toString call on the DAML value
                Component c = super.getListCellRendererComponent( list,
                                                                  m_viewer.renderDAMLObjectName( value ),
                                                                  index, isSel, hasFocus );

                // bold immediate super-classes
                for (Iterator i = m_class.getDefinedProperties( false );  i.hasNext(); ) {
                    if (i.next().equals( value )) {
                        // an immediate parent
                        if (c instanceof JLabel) {
                            Font f = ((JLabel) c).getFont();
                            ((JLabel) c).setFont( new Font( f.getName(), Font.BOLD, f.getSize() ) );
                            break;
                        }
                    }
                }

                return c;
            }
        };
        border1 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder1 = new TitledBorder(border1,"Super-classes");
        border2 = BorderFactory.createCompoundBorder(titledBorder1,BorderFactory.createEmptyBorder(1,1,1,1));
        border3 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder2 = new TitledBorder(border3,"Class comment");
        border4 = BorderFactory.createCompoundBorder(titledBorder2,BorderFactory.createEmptyBorder(1,1,1,1));
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setTitle("Class Inspector");
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                this_windowClosing(e);
            }
        });
        this.getContentPane().setLayout(borderLayout1);
        pnlDescription.setLayout(gridLayout1);
        pnlValues.setLayout(gridLayout2);
        pnlDescription.setBorder(border4);
        pnlPropertyGroup.setLayout(gridLayout6);
        pnlEquivs.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Equivalent classes"),BorderFactory.createEmptyBorder(1,1,1,1)));
        pnlEquivs.setLayout(gridLayout5);
        pnlRelations.setLayout(gridLayout7);
        pnlSubs.setLayout(gridLayout3);
        pnlSupers.setLayout(gridLayout4);
        pnlSupers.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Super-classes"),BorderFactory.createEmptyBorder(1,1,1,1)));
        pnlSubs.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Sub-classes"),BorderFactory.createEmptyBorder(1,1,1,1)));
        pnlInstances.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Instances"),BorderFactory.createEmptyBorder(1,1,1,1)));
        pnlInstances.setLayout(gridLayout8);
        gridLayout6.setRows(2);
        gridLayout6.setColumns(1);
        pnlProperties.setLayout(borderLayout3);
        pnlProperties.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Class properties"),BorderFactory.createEmptyBorder(1,1,1,1)));
        gridLayout7.setRows(2);
        gridLayout7.setColumns(1);
        lstInstances.setCellRenderer(damlRenderer);
        lstInstances.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstInstances.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstInstances_mouseClicked(e);
            }
        });
        lstSubClasses.setCellRenderer(subClassListRenderer);
        lstSubClasses.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstSubClasses.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstSubClasses_mouseClicked(e);
            }
        });
        lstSuperClasses.setCellRenderer(superClassListRenderer);
        lstSuperClasses.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstSuperClasses.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstSuperClasses_mouseClicked(e);
            }
        });
        lstEquivs.setCellRenderer(damlRenderer);
        lstEquivs.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstEquivs.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstEquivs_mouseClicked(e);
            }
        });
        lstProperties.setCellRenderer(propertiesListRenderer);
        lstProperties.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstProperties.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstProperties_mouseClicked(e);
            }
        });
        txtDescription.setWrapStyleWord(true);
        txtDescription.setText("Sorry, no description available");
        txtDescription.setEditable(false);
        txtDescription.setFont(new java.awt.Font("Dialog", 0, 11));
        this.getContentPane().add(pnlDescription, BorderLayout.NORTH);
        pnlDescription.add(jScrollPane6, null);
        jScrollPane6.getViewport().add(txtDescription, null);
        this.getContentPane().add(pnlValues, BorderLayout.CENTER);
        pnlValues.add(pnlPropertyGroup, null);
        pnlPropertyGroup.add(pnlProperties, null);
        pnlProperties.add(jScrollPane1, BorderLayout.CENTER);
        jScrollPane1.getViewport().add(lstProperties, null);
        pnlPropertyGroup.add(pnlEquivs, null);
        pnlEquivs.add(jScrollPane2, null);
        jScrollPane2.getViewport().add(lstEquivs, null);
        pnlValues.add(pnlRelations, null);
        pnlRelations.add(pnlSupers, null);
        pnlSupers.add(jScrollPane3, null);
        jScrollPane3.getViewport().add(lstSuperClasses, null);
        pnlRelations.add(pnlSubs, null);
        pnlSubs.add(jScrollPane4, null);
        jScrollPane4.getViewport().add(lstSubClasses, null);
        pnlValues.add(pnlInstances, null);
        pnlInstances.add(jScrollPane5, null);
        jScrollPane5.getViewport().add(lstInstances, null);
    }

    /**
     * If the user clicks on a class, open an inspector
     */
    void lstSuperClasses_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstSuperClasses.getSelectedValue() );
    }

    void this_windowClosing(WindowEvent e) {
         m_viewer.getFrame().removeWindowMenuEntry( m_mnuItem );
         m_viewer.removeInspector( this );
    }

    void lstProperties_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstProperties.getSelectedValue() );
    }

    void lstSubClasses_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstSubClasses.getSelectedValue() );
    }

    void lstEquivs_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstEquivs.getSelectedValue() );
    }

    void lstInstances_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstInstances.getSelectedValue() );
    }


    //==============================================================================
    // Inner class definitions
    //==============================================================================


}
