/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Ian Dickinson, HP Labs Bristol
 * Author email       Ian_Dickinson@hp.com
 * Package
 * Created            4 Jan 2002
 * Filename           $RCSfile: InstanceInspector.java,v $
 * Revision           $Revision: 1.1 $
 * Release status     Experimental. Do not distribute. $State: Exp $
 *
 * Last modified on   $Date: 2002/01/10 14:18:52 $
 *               by   $Author: ijd $
 *
 * (c) Copyright Hewlett-Packard Company 2002
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

// Package
///////////////
package examples.daml.ex1;


// Imports
///////////////
import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.util.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import java.awt.event.*;


/**
 * Value inspector for DAML instances
 *
 * @author Ian Dickinson, HP Labs (<a href="mailto:Ian_Dickinson@hp.com">email</a>)
 * @version CVS info: $Id: InstanceInspector.java,v 1.1 2002/01/10 14:18:52 ijd Exp $
 */
public class InstanceInspector
    extends JFrame
    implements DAMLInspector
{

    // Constants
    //////////////////////////////////


    // Static variables
    //////////////////////////////////


    // Instance variables
    //////////////////////////////////

    /** The DAML class this inspector is showing */
    private DAMLInstance m_instance = null;

    /** The application object */
    private DAMLViewer m_viewer = null;

    /** My menu entry on the window menu */
    private JMenuItem m_mnuItem = null;


    // list models
    private DAMLListModel m_classesModel = null;

    // table model
    private DAMLTableModel m_propertiesModel = null;

    /* JBuilder generated instance variables for GUI layout */
    BorderLayout borderLayout1 = new BorderLayout();
    JPanel pnlDescription = new JPanel();
    GridLayout gridLayout1 = new GridLayout();
    JPanel pnlValues = new JPanel();
    GridLayout gridLayout2 = new GridLayout();
    Border border1;
    TitledBorder titledBorder1;
    Border border2;
    JScrollPane jScrollPane6 = new JScrollPane();
    JTextArea txtDescription = new JTextArea();
    Border border3;
    TitledBorder titledBorder2;
    Border border4;
    JSplitPane spltInstanceValues = new JSplitPane();
    JScrollPane jScrollPane5 = new JScrollPane();
    GridLayout gridLayout8 = new GridLayout();
    JPanel pnlProperties = new JPanel();
    JPanel pnlClasses = new JPanel();
    GridLayout gridLayout6 = new GridLayout();
    Border border5;
    TitledBorder titledBorder3;
    Border border6;
    Border border7;
    TitledBorder titledBorder4;
    Border border8;
    JScrollPane jScrollPane1 = new JScrollPane();
    JList lstClasses = new JList();
    JTable tblProperties = new JTable();


    // Constructors
    //////////////////////////////////

    /**
     * Construct an instance inspector for the given DAMLInstance
     */
    public InstanceInspector( DAMLInstance inst, DAMLViewer viewer ) {
        m_instance = inst;
        m_viewer = viewer;

        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }


    // External signature methods
    //////////////////////////////////

    /**
     * Make the inspector visible for the first time
     * @see makeVisible
     */
    public void showInspector() {
        // initialise the inspector's data
        setTitle( "Instance Inspector for " + m_viewer.renderDAMLObjectName( m_instance ) );
        try {
            String desc = m_instance.prop_comment().getValue().getString();
            if (desc.length() > 0) {
                txtDescription.setText( desc );
            }
        }
        catch (RDFException ignore) {}
        catch (NullPointerException ignore) {}

        // set up the list models
        m_classesModel = new DAMLListModel( m_viewer, (DAMLModel) m_instance.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_instance.getRDFTypes( true );
                                                }
                                            },
                                            true );
        m_propertiesModel = new DAMLTableModel( (DAMLModel) m_instance.getModel(),
                                                    new DAMLStatementFilter() {
                                                        public StmtIterator filterDAMLStatements( DAMLModel m ) {
                                                            try {
                                                                return m_instance.listProperties();
                                                            }
                                                            catch (RDFException ignore) {
                                                                // TODO: should really log the exception here
                                                                return null;
                                                            }
                                                        }
                                                    } );

        lstClasses.setModel( m_classesModel );
        tblProperties.setModel( m_propertiesModel );

        // make myself visible
        setSize( 500, 350 );
        setVisible( true );
    }


    /**
     * Answer the title of the inspector
     */
    public String getMenuTitle() {
        return "Instance " + m_viewer.renderDAMLObjectName( m_instance );
    }


    /**
     * Ensure that the inspector visible (even if iconified) and has focus
     */
    public void reveal() {
        setState( Frame.NORMAL );
        show();
        toFront();
        requestFocus();
    }


    /**
     * Set a reference to my entry in the window menu
     *
     */
    public void setWindowMenuEntry( JMenuItem mnu ) {
        m_mnuItem = mnu;
    }


    public void dispose() {
        m_viewer.getFrame().removeWindowMenuEntry( m_mnuItem );
        super.dispose();
    }



    // Internal implementation methods
    //////////////////////////////////

    /**
     * GUI layout - generated by JBuilder's design tool
     */
    private void jbInit() throws Exception {
        ListCellRenderer damlRenderer = new DefaultListCellRenderer() {
            public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
                // override the toString call on the DAML value
                return super.getListCellRendererComponent( list,
                                                           m_viewer.renderDAMLObjectName( value ),
                                                           index, isSel, hasFocus );
            }
        };
        TableCellRenderer damlCellRenderer = new DefaultTableCellRenderer() {
            public Component getTableCellRendererComponent( JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column ) {
                // override the toString call on the DAML value
                return super.getTableCellRendererComponent( table,
                                                           m_viewer.renderDAMLObjectName( value ),
                                                           isSelected, hasFocus, row, column );
            }
        };
        border1 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder1 = new TitledBorder(border1,"Super-classes");
        border2 = BorderFactory.createCompoundBorder(titledBorder1,BorderFactory.createEmptyBorder(1,1,1,1));
        border3 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder2 = new TitledBorder(border3,"Class comment");
        border4 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Instance comment"),BorderFactory.createEmptyBorder(1,1,1,1));
        border5 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder3 = new TitledBorder(border5,"Instance properties");
        border6 = BorderFactory.createCompoundBorder(titledBorder3,BorderFactory.createEmptyBorder(1,1,1,1));
        border7 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder4 = new TitledBorder(border7,"Classes");
        border8 = BorderFactory.createCompoundBorder(titledBorder4,BorderFactory.createEmptyBorder(1,1,1,1));
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setTitle("Class Inspector");
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                this_windowClosing(e);
            }
        });
        this.getContentPane().setLayout(borderLayout1);
        pnlDescription.setLayout(gridLayout1);
        pnlValues.setLayout(gridLayout2);
        pnlDescription.setBorder(border4);
        txtDescription.setWrapStyleWord(true);
        txtDescription.setText("Sorry, no description available");
        txtDescription.setEditable(false);
        txtDescription.setFont(new java.awt.Font("Dialog", 0, 11));
        pnlProperties.setBorder(border6);
        pnlProperties.setLayout(gridLayout8);
        pnlClasses.setLayout(gridLayout6);
        gridLayout6.setColumns(1);
        spltInstanceValues.setBottomComponent(null);
        spltInstanceValues.setTopComponent(null);
        lstClasses.setCellRenderer(damlRenderer);
        lstClasses.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstClasses.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstClasses_mouseClicked(e);
            }
        });
        jScrollPane1.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Classes"),BorderFactory.createEmptyBorder(1,1,1,1)));
        jScrollPane5.getViewport().setBackground(Color.white);
        tblProperties.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                tblProperties_mouseClicked(e);
            }
        });
        this.getContentPane().add(pnlDescription, BorderLayout.NORTH);
        pnlDescription.add(jScrollPane6, null);
        jScrollPane6.getViewport().add(txtDescription, null);
        this.getContentPane().add(pnlValues, BorderLayout.CENTER);
        pnlValues.add(spltInstanceValues, null);
        spltInstanceValues.add(pnlProperties, JSplitPane.BOTTOM);
        pnlProperties.add(jScrollPane5, null);
        jScrollPane5.getViewport().add(tblProperties, null);
        spltInstanceValues.add(pnlClasses, JSplitPane.TOP);
        pnlClasses.add(jScrollPane1, null);
        jScrollPane1.getViewport().add(lstClasses, null);
        spltInstanceValues.setDividerLocation(200);
        tblProperties.setDefaultRenderer( Resource.class, damlCellRenderer );
    }


    void this_windowClosing(WindowEvent e) {
         m_viewer.getFrame().removeWindowMenuEntry( m_mnuItem );
         m_viewer.removeInspector( this );
    }

    void lstClasses_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstClasses.getSelectedValue() );
    }

    void tblProperties_mouseClicked(MouseEvent e) {
        int row = tblProperties.getSelectedRow();
        Object val = tblProperties.getModel().getValueAt( row, 1 );
        if (val instanceof DAMLCommon) {
            m_viewer.inspect( (DAMLCommon) val );
        }
    }


    //==============================================================================
    // Inner class definitions
    //==============================================================================


}
