/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Ian Dickinson, HP Labs Bristol
 * Author email       Ian_Dickinson@hp.com
 * Package
 * Created            4 Jan 2002
 * Filename           $RCSfile: PropertyInspector.java,v $
 * Revision           $Revision: 1.1 $
 * Release status     Experimental. Do not distribute. $State: Exp $
 *
 * Last modified on   $Date: 2002/01/10 14:18:52 $
 *               by   $Author: ijd $
 *
 * (c) Copyright Hewlett-Packard Company 2002
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

// Package
///////////////
package examples.daml.ex1;


// Imports
///////////////
import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.util.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.jena.daml.common.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import java.awt.event.*;



/**
 * Value inspector for DAML properties
 *
 * @author Ian Dickinson, HP Labs (<a href="mailto:Ian_Dickinson@hp.com">email</a>)
 * @version CVS info: $Id: PropertyInspector.java,v 1.1 2002/01/10 14:18:52 ijd Exp $
 */
public class PropertyInspector
    extends JFrame
    implements DAMLInspector
{

    // Constants
    //////////////////////////////////


    // Static variables
    //////////////////////////////////


    // Instance variables
    //////////////////////////////////

    /** The DAML property this inspector is showing */
    private DAMLProperty m_prop = null;

    /** The application object */
    private DAMLViewer m_viewer = null;

    /** My menu entry on the window menu */
    private JMenuItem m_mnuItem = null;


    // list models
    private DAMLListModel m_rangeModel = null;
    private DAMLListModel m_equivsModel = null;
    private DAMLListModel m_supersModel = null;
    private DAMLListModel m_subsModel = null;
    private DAMLListModel m_domainModel = null;

    /* JBuilder generated instance variables for GUI layout */
    BorderLayout borderLayout1 = new BorderLayout();
    JPanel pnlDescription = new JPanel();
    GridLayout gridLayout1 = new GridLayout();
    JPanel pnlValues = new JPanel();
    GridLayout gridLayout2 = new GridLayout();
    JPanel pnlPropertyGroup = new JPanel();
    JPanel pnlInstances = new JPanel();
    JPanel pnlRelations = new JPanel();
    JPanel pnlEquivs = new JPanel();
    JPanel pnlSupers = new JPanel();
    JPanel pnlSubs = new JPanel();
    GridLayout gridLayout3 = new GridLayout();
    GridLayout gridLayout4 = new GridLayout();
    Border border1;
    TitledBorder titledBorder1;
    Border border2;
    GridLayout gridLayout5 = new GridLayout();
    GridLayout gridLayout6 = new GridLayout();
    JScrollPane jScrollPane2 = new JScrollPane();
    JList lstRange = new JList();
    JPanel pnlProperties = new JPanel();
    BorderLayout borderLayout3 = new BorderLayout();
    JScrollPane jScrollPane1 = new JScrollPane();
    JList lstDomain = new JList();
    JScrollPane jScrollPane3 = new JScrollPane();
    JList lstSuperProperties = new JList();
    JScrollPane jScrollPane4 = new JScrollPane();
    JList lstSubProperties = new JList();
    GridLayout gridLayout7 = new GridLayout();
    GridLayout gridLayout8 = new GridLayout();
    JScrollPane jScrollPane5 = new JScrollPane();
    JList lstEquivs = new JList();
    JScrollPane jScrollPane6 = new JScrollPane();
    JTextArea txtDescription = new JTextArea();
    Border border3;
    TitledBorder titledBorder2;
    Border border4;
    Border border5;
    TitledBorder titledBorder3;
    Border border6;
    Border border7;
    TitledBorder titledBorder4;
    Border border8;
    Border border9;
    TitledBorder titledBorder5;
    Border border10;
    Border border11;
    TitledBorder titledBorder6;
    Border border12;
    Border border13;
    TitledBorder titledBorder7;
    Border border14;


    // Constructors
    //////////////////////////////////

    /**
     * Construct a property inspector for the given DAMLProperty
     */
    public PropertyInspector( DAMLProperty prop, DAMLViewer viewer ) {
        m_prop = prop;
        m_viewer = viewer;

        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }


    // External signature methods
    //////////////////////////////////

    /**
     * Make the inspector visible for the first time
     * @see makeVisible
     */
    public void showInspector() {
        // initialise the inspector's data
        setTitle( "Property Inspector for " + m_viewer.renderDAMLObjectName( m_prop ) );
        try {
            String desc = m_prop.prop_comment().getValue().getString();
            if (desc.length() > 0) {
                txtDescription.setText( desc );
            }
        }
        catch (RDFException ignore) {}
        catch (NullPointerException ignore) {}

        // set up the list models
        m_equivsModel = new DAMLListModel( m_viewer, (DAMLModel) m_prop.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_prop.getEquivalenceSet();
                                                }
                                            },
                                            true );
        m_subsModel = new DAMLListModel( m_viewer, (DAMLModel) m_prop.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_prop.getSubProperties();
                                                }
                                            },
                                            true );
        m_supersModel = new DAMLListModel( m_viewer, (DAMLModel) m_prop.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_prop.getSuperProperties();
                                                }
                                            },
                                            true );
        m_domainModel = new DAMLListModel( m_viewer, (DAMLModel) m_prop.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_prop.getDomainClasses();
                                                }
                                            },
                                            true );
        m_rangeModel = new DAMLListModel( m_viewer, (DAMLModel) m_prop.getModel(),
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m_prop.getRangeClasses();
                                                }
                                            },
                                            true );

        lstEquivs.setModel( m_equivsModel );
        lstSubProperties.setModel( m_subsModel );
        lstSuperProperties.setModel( m_supersModel );
        lstDomain.setModel( m_domainModel );
        lstRange.setModel( m_rangeModel );

        // make myself visible
        setSize( 500, 350 );
        setVisible( true );
    }


    /**
     * Answer the title of the inspector
     */
    public String getMenuTitle() {
        return "Property " + m_viewer.renderDAMLObjectName( m_prop );
    }


    /**
     * Ensure that the inspector visible (even if iconified) and has focus
     */
    public void reveal() {
        setState( Frame.NORMAL );
        show();
        toFront();
        requestFocus();
    }


    /**
     * Set a reference to my entry in the window menu
     *
     */
    public void setWindowMenuEntry( JMenuItem mnu ) {
        m_mnuItem = mnu;
    }


    public void dispose() {
        m_viewer.getFrame().removeWindowMenuEntry( m_mnuItem );
        super.dispose();
    }


    // Internal implementation methods
    //////////////////////////////////

    /**
     * GUI layout - generated by JBuilder's design tool
     */
    private void jbInit() throws Exception {
        ListCellRenderer damlRenderer = new DefaultListCellRenderer() {
            public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
                // override the toString call on the DAML value
                return super.getListCellRendererComponent( list,
                                                           m_viewer.renderDAMLObjectName( value ),
                                                           index, isSel, hasFocus );
            }
        };
        ListCellRenderer superPropListRenderer = new DefaultListCellRenderer() {
            public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
                // override the toString call on the DAML value
                Component c = super.getListCellRendererComponent( list,
                                                                  m_viewer.renderDAMLObjectName( value ),
                                                                  index, isSel, hasFocus );

                // bold immediate super-props
                for (Iterator i = m_prop.getSuperProperties( false );  i.hasNext(); ) {
                    if (i.next().equals( value )) {
                        // an immediate parent
                        if (c instanceof JLabel) {
                            Font f = ((JLabel) c).getFont();
                            ((JLabel) c).setFont( new Font( f.getName(), Font.BOLD, f.getSize() ) );
                            break;
                        }
                    }
                }

                return c;
            }
        };
        ListCellRenderer subPropListRenderer = new DefaultListCellRenderer() {
            public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
                // override the toString call on the DAML value
                Component c = super.getListCellRendererComponent( list,
                                                                  m_viewer.renderDAMLObjectName( value ),
                                                                  index, isSel, hasFocus );

                // bold immediate super-props
                for (Iterator i = m_prop.getSubProperties( false );  i.hasNext(); ) {
                    if (i.next().equals( value )) {
                        // an immediate parent
                        if (c instanceof JLabel) {
                            Font f = ((JLabel) c).getFont();
                            ((JLabel) c).setFont( new Font( f.getName(), Font.BOLD, f.getSize() ) );
                            break;
                        }
                    }
                }

                return c;
            }
        };

        border1 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder1 = new TitledBorder(border1,"Super-Properties");
        border2 = BorderFactory.createCompoundBorder(titledBorder1,BorderFactory.createEmptyBorder(1,1,1,1));
        border3 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder2 = new TitledBorder(border3,"Property comment");
        border4 = BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Property comment"),BorderFactory.createEmptyBorder(1,1,1,1));
        border5 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder3 = new TitledBorder(border5,"Domain classes");
        border6 = BorderFactory.createCompoundBorder(titledBorder3,BorderFactory.createEmptyBorder(1,1,1,1));
        border7 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder4 = new TitledBorder(border7,"Range classes");
        border8 = BorderFactory.createCompoundBorder(titledBorder4,BorderFactory.createEmptyBorder(1,1,1,1));
        border9 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder5 = new TitledBorder(border9,"Super-properties");
        border10 = BorderFactory.createCompoundBorder(titledBorder5,BorderFactory.createEmptyBorder(1,1,1,1));
        border11 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder6 = new TitledBorder(border11,"Sub-properties");
        border12 = BorderFactory.createCompoundBorder(titledBorder6,BorderFactory.createEmptyBorder(1,1,1,1));
        border13 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder7 = new TitledBorder(border13,"Equivalent properties");
        border14 = BorderFactory.createCompoundBorder(titledBorder7,BorderFactory.createEmptyBorder(1,1,1,1));
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setTitle("Property Inspector");
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                this_windowClosing(e);
            }
        });
        this.getContentPane().setLayout(borderLayout1);
        pnlDescription.setLayout(gridLayout1);
        pnlValues.setLayout(gridLayout2);
        pnlDescription.setBorder(border4);
        pnlPropertyGroup.setLayout(gridLayout6);
        pnlEquivs.setBorder(border8);
        pnlEquivs.setLayout(gridLayout5);
        pnlRelations.setLayout(gridLayout7);
        pnlSubs.setLayout(gridLayout3);
        pnlSupers.setLayout(gridLayout4);
        pnlSupers.setBorder(border10);
        pnlSubs.setBorder(border12);
        pnlInstances.setBorder(border14);
        pnlInstances.setLayout(gridLayout8);
        gridLayout6.setRows(2);
        gridLayout6.setColumns(1);
        pnlProperties.setLayout(borderLayout3);
        pnlProperties.setBorder(border6);
        gridLayout7.setRows(2);
        gridLayout7.setColumns(1);
        lstEquivs.setCellRenderer(damlRenderer);
        lstEquivs.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstEquivs.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstEquivs_mouseClicked(e);
            }
        });
        lstSubProperties.setCellRenderer(subPropListRenderer);
        lstSubProperties.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstSubProperties.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstSubProperties_mouseClicked(e);
            }
        });
        lstSuperProperties.setCellRenderer(superPropListRenderer);
        lstSuperProperties.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstSuperProperties.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstSuperProperties_mouseClicked(e);
            }
        });
        lstRange.setCellRenderer(damlRenderer);
        lstRange.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstRange.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstRange_mouseClicked(e);
            }
        });
        lstDomain.setCellRenderer(damlRenderer);
        lstDomain.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstDomain.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstDomain_mouseClicked(e);
            }
        });
        txtDescription.setWrapStyleWord(true);
        txtDescription.setText("Sorry, no description available");
        txtDescription.setEditable(false);
        txtDescription.setFont(new java.awt.Font("Dialog", 0, 11));
        this.getContentPane().add(pnlDescription, BorderLayout.NORTH);
        pnlDescription.add(jScrollPane6, null);
        jScrollPane6.getViewport().add(txtDescription, null);
        this.getContentPane().add(pnlValues, BorderLayout.CENTER);
        pnlValues.add(pnlPropertyGroup, null);
        pnlPropertyGroup.add(pnlProperties, null);
        pnlProperties.add(jScrollPane1, BorderLayout.CENTER);
        jScrollPane1.getViewport().add(lstDomain, null);
        pnlPropertyGroup.add(pnlEquivs, null);
        pnlEquivs.add(jScrollPane2, null);
        jScrollPane2.getViewport().add(lstRange, null);
        pnlValues.add(pnlRelations, null);
        pnlRelations.add(pnlSupers, null);
        pnlSupers.add(jScrollPane3, null);
        jScrollPane3.getViewport().add(lstSuperProperties, null);
        pnlRelations.add(pnlSubs, null);
        pnlSubs.add(jScrollPane4, null);
        jScrollPane4.getViewport().add(lstSubProperties, null);
        pnlValues.add(pnlInstances, null);
        pnlInstances.add(jScrollPane5, null);
        jScrollPane5.getViewport().add(lstEquivs, null);
    }

    /**
     * If the user clicks on a prop, open an inspector
     */
    void lstSuperProperties_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstSuperProperties.getSelectedValue() );
    }

    void this_windowClosing(WindowEvent e) {
         m_viewer.getFrame().removeWindowMenuEntry( m_mnuItem );
         m_viewer.removeInspector( this );
    }

    void lstDomain_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstDomain.getSelectedValue() );
    }

    void lstRange_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstRange.getSelectedValue() );
    }

    void lstSubProperties_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstSubProperties.getSelectedValue() );
    }

    void lstEquivs_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstEquivs.getSelectedValue() );
    }


    //==============================================================================
    // Inner class definitions
    //==============================================================================


}
