/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Ian Dickinson, HP Labs Bristol
 * Author email       Ian_Dickinson@hp.com
 * Package
 * Created            4 Jan 2002
 * Filename           $RCSfile: ViewerFrame.java,v $
 * Revision           $Revision: 1.2 $
 * Release status     Experimental. Do not distribute. $State: Exp $
 *
 * Last modified on   $Date: 2002/01/25 18:17:37 $
 *               by   $Author: ijd $
 *
 * (c) Copyright Hewlett-Packard Company 2002
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

// Package
///////////////
package examples.daml.ex1;


// Imports
///////////////
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.jena.daml.common.*;
import com.hp.hpl.mesa.rdf.jena.model.*;



/**
 * SwingUI frame for the main viewer of a DAML ontology.
 *
 * @author Ian Dickinson, HP Labs (<a href="mailto:Ian_Dickinson@hp.com">email</a>)
 * @version CVS info: $Id: ViewerFrame.java,v 1.2 2002/01/25 18:17:37 ijd Exp $
 */
public class ViewerFrame
    extends JFrame
{

    // Constants
    //////////////////////////////////


    // Static variables
    //////////////////////////////////


    // Instance variables
    //////////////////////////////////

    /** The model that is being viewed */
    private DAMLModel m_model = null;

    /** The currently loaded document's URI */
    private String m_currentURI = null;

    /** Reference to the main viewer app */
    private DAMLViewer m_viewer = null;


    // list models

    /** List model for classes list */
    private DAMLListModel m_classesModel = null;

    /** List model for properties list */
    private DAMLListModel m_propertiesModel = null;

    /** List model for instances list */
    private DAMLListModel m_instancesModel = null;


    /** Generated by JBuilder UI designer */
    BorderLayout borderLayout1 = new BorderLayout();
    JMenuBar mnuMain = new JMenuBar();
    JMenu mnuFile = new JMenu();
    JMenuItem mnuItemOpenURL = new JMenuItem();
    JMenuItem mnuItemExit = new JMenuItem();
    JMenu mnuWindow = new JMenu();
    JMenuItem mnuItemCloseAll = new JMenuItem();
    JPanel pnlListings = new JPanel();
    JPanel pnlClasses = new JPanel();
    JPanel pnlProperties = new JPanel();
    JPanel pnlInstances = new JPanel();
    TitledBorder titledBorder1;
    Border border1;
    TitledBorder titledBorder2;
    Border border2;
    Border border3;
    TitledBorder titledBorder3;
    Border border4;
    JMenu mnuPreferences = new JMenu();
    JCheckBoxMenuItem mnuItemProcessImports = new JCheckBoxMenuItem();
    JCheckBoxMenuItem mnuItemUseImportBlocking = new JCheckBoxMenuItem();
    JMenuItem mnuItemProperties = new JMenuItem();
    GridLayout gridLayout1 = new GridLayout();
    GridLayout gridLayout2 = new GridLayout();
    GridLayout gridLayout3 = new GridLayout();

    ListCellRenderer damlRenderer = new DefaultListCellRenderer() {
        public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSel, boolean hasFocus ) {
            // override the toString call on the DAML value
            return super.getListCellRendererComponent( list,
                                                       m_viewer.renderDAMLObjectName( value ),
                                                       index, isSel, hasFocus );
        }
    };
    JCheckBoxMenuItem mnuItemShowAnonymous = new JCheckBoxMenuItem();
    JScrollPane jScrollPane1 = new JScrollPane();
    JList lstAllInstances = new JList();
    JScrollPane jScrollPane2 = new JScrollPane();
    JList lstAllProperties = new JList();
    JScrollPane jScrollPane3 = new JScrollPane();
    JList lstAllClasses = new JList();
    GridLayout gridLayout4 = new GridLayout();


    // Constructors
    //////////////////////////////////

    /**
     * Construct the frame for the main ontology viewer
     */
    public ViewerFrame( DAMLViewer viewer ) {
        m_viewer = viewer;

        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    // External signature methods
    //////////////////////////////////

    /**
     * Load a DAML document into the viewed model, given
     * its URI
     *
     * @param uri The URI to load, as a string
     */
    public void loadURL( String uri ) {
        Cursor c = getGlassPane().getCursor();

        try {
            // show the hourglass
            getGlassPane().setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
            getGlassPane().setVisible( true );

            boolean failed = false;

            // save the URI for future use
            m_currentURI = uri;
            m_model = new DAMLModelImpl();

            // first we load the document into the model
            try {
                m_model.getLoader().setLoadImportedOntologies( m_viewer.getLoadImports() );
                m_model.getLoader().setUseImportBlocking( m_viewer.getUseImportBlocking() );
                m_model.read( uri );
            }
            catch (RDFException e) {
                failed = true;
            }

            // did it work?
            if (failed || !m_model.getLoadSuccessful()) {
                // load failure - let the user know what happened
                // TODO: give more feedback on the reason for the failure
                JOptionPane.showMessageDialog( this, "Failed to load DAML source from " + uri, "", JOptionPane.ERROR_MESSAGE );
            }
            else {
                // yes it worked - update the display
                removeAllInspectors();
                reloadData();

                // update the window title
                setTitle( "Jena DAML Viewer (" + uri + ")" );
            }
        }
        finally {
            getGlassPane().setCursor( c );
            getGlassPane().setVisible( false );
        }
    }


    /**
     * Refresh the UI since the underlying model has changed
     */
    public void reloadData() {
        // create models for the display lists
        m_classesModel = new DAMLListModel( m_viewer, m_model,
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m.listDAMLClasses();
                                                }
                                            },
                                            true );
        m_propertiesModel = new DAMLListModel( m_viewer, m_model,
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m.listDAMLProperties();
                                                }
                                            },
                                            true );
        m_instancesModel = new DAMLListModel( m_viewer, m_model,
                                            new DAMLFilter() {
                                                public Iterator filterDAMLValues( DAMLModel m ) {
                                                    return m.listDAMLInstances();
                                                }
                                            },
                                            true );

        // connect them to the lists themselves
        lstAllClasses.setModel( m_classesModel );
        lstAllProperties.setModel( m_propertiesModel );
        lstAllInstances.setModel( m_instancesModel );
    }


    /**
     * Add an item to the window menu
     */
    public JMenuItem addWindowMenuEntry( DAMLInspector inspector ) {
        JMenuItem mnu = new JMenuItem( inspector.getMenuTitle() );
        mnu.addActionListener( new WindowMenuActionListener( inspector ) );

        mnuWindow.add( mnu );

        return mnu;
    }


    /**
     * Remove a window menu entry
     */
    public void removeWindowMenuEntry( JMenuItem item ) {
        mnuWindow.remove( item );
    }



    // Internal implementation methods
    //////////////////////////////////

    /**
     * Initialise the GUI elements. Generated by JBuilder
     * UI designer
     */
    private void jbInit() throws Exception {
        titledBorder1 = new TitledBorder("");
        border1 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder2 = new TitledBorder(border1,"All Properties");
        border2 = BorderFactory.createCompoundBorder(titledBorder2,BorderFactory.createEmptyBorder(2,2,2,2));
        border3 = BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134));
        titledBorder3 = new TitledBorder(border3,"All Classes");
        border4 = BorderFactory.createCompoundBorder(titledBorder3,BorderFactory.createEmptyBorder(2,2,2,2));
        this.setDefaultCloseOperation(3);
        this.setTitle("Jena DAML Viewer");
        this.getContentPane().setLayout(borderLayout1);
        mnuItemOpenURL.setText("Open URL ...");
        mnuItemOpenURL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mnuItemOpenURL_actionPerformed(e);
            }
        });
        mnuFile.setText("File");
        mnuItemExit.setText("Exit");
        mnuItemExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mnuItemExit_actionPerformed(e);
            }
        });
        mnuWindow.setText("Window");
        mnuItemCloseAll.setActionCommand("");
        mnuItemCloseAll.setText("Close all inspectors");
        mnuItemCloseAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mnuItemCloseAll_actionPerformed(e);
            }
        });
        pnlListings.setLayout(gridLayout4);
        pnlClasses.setBorder(border4);
        pnlClasses.setLayout(gridLayout1);
        pnlProperties.setBorder(border2);
        pnlProperties.setLayout(gridLayout2);
        pnlInstances.setBorder(BorderFactory.createCompoundBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"All Instances"),BorderFactory.createEmptyBorder(2,2,2,2)));
        pnlInstances.setLayout(gridLayout3);
        mnuPreferences.setText("Preferences");
        mnuItemProcessImports.setText("Process imports");
        mnuItemProcessImports.setSelected(true);
        mnuItemProcessImports.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mnuItemProcessImports_actionPerformed(e);
            }
        });
        mnuItemUseImportBlocking.setText("Use import blocking");
        mnuItemUseImportBlocking.setSelected(true);
        mnuItemUseImportBlocking.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mnuItemUseImportBlocking_actionPerformed(e);
            }
        });
        mnuItemProperties.setText("Properties");
        mnuItemProperties.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mnuItemProperties_actionPerformed(e);
            }
        });
        mnuItemShowAnonymous.setText("Show anonymous");
        mnuItemShowAnonymous.setSelected(true);
        mnuItemShowAnonymous.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mnuItemShowAnonymous_actionPerformed(e);
            }
        });
        lstAllInstances.setCellRenderer(damlRenderer);
        lstAllInstances.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstAllInstances.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstAllInstances_mouseClicked(e);
            }
        });
        lstAllProperties.setCellRenderer(damlRenderer);
        lstAllProperties.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstAllProperties.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstAllProperties_mouseClicked(e);
            }
        });
        lstAllClasses.setCellRenderer(damlRenderer);
        lstAllClasses.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstAllClasses.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                lstAllClasses_mouseClicked(e);
            }
        });
        gridLayout4.setColumns(3);
        mnuMain.add(mnuFile);
        mnuMain.add(mnuPreferences);
        mnuMain.add(mnuWindow);
        mnuFile.add(mnuItemOpenURL);
        mnuFile.add(mnuItemProperties);
        mnuFile.addSeparator();
        mnuFile.add(mnuItemExit);
        mnuWindow.add(mnuItemCloseAll);
        mnuWindow.addSeparator();
        this.getContentPane().add( mnuMain, BorderLayout.NORTH );
        this.getContentPane().add(pnlListings, BorderLayout.CENTER);
        pnlListings.add(pnlClasses, null);
        pnlClasses.add(jScrollPane3, null);
        jScrollPane3.getViewport().add(lstAllClasses, null);
        pnlListings.add(pnlProperties, null);
        pnlProperties.add(jScrollPane2, null);
        jScrollPane2.getViewport().add(lstAllProperties, null);
        pnlListings.add(pnlInstances, null);
        pnlInstances.add(jScrollPane1, null);
        jScrollPane1.getViewport().add(lstAllInstances, null);
        mnuPreferences.add(mnuItemProcessImports);
        mnuPreferences.add(mnuItemUseImportBlocking);
        mnuPreferences.add(mnuItemShowAnonymous);
    }


    /**
     * Remove all inpsector windows
     */
    private void removeAllInspectors() {
    }


    /**
     * Shut down the app when the user clicks exit
     */
    void mnuItemExit_actionPerformed(ActionEvent e) {
        // crude, but effective
        System.exit( 0 );
    }

    /**
     * To show or not to show anonymous values
     */
    void mnuItemShowAnonymous_actionPerformed(ActionEvent e) {
        m_viewer.setShowAnonymous( mnuItemShowAnonymous.isSelected() );
        reloadData();
    }

    /**
     * Choose whether to load imported ontologies
     */
    void mnuItemProcessImports_actionPerformed(ActionEvent e) {
        m_viewer.setLoadImports( mnuItemProcessImports.isSelected() );
        loadURL( m_currentURI );
    }

    /**
     * Choose whether to block well-known imports (e.g. DAML+OIL)
     */
    void mnuItemUseImportBlocking_actionPerformed(ActionEvent e) {
        m_viewer.setUseImportBlocking( mnuItemUseImportBlocking.isSelected() );
        loadURL( m_currentURI );
    }

    /**
     * On clicking on the class list, show a class inspector
     */
    void lstAllClasses_mouseClicked(MouseEvent e) {
        m_viewer.inspect( (DAMLCommon) lstAllClasses.getSelectedValue() );
    }


    void lstAllProperties_mouseClicked(MouseEvent e) {
         m_viewer.inspect( (DAMLCommon) lstAllProperties.getSelectedValue() );
    }

    void lstAllInstances_mouseClicked(MouseEvent e) {
         m_viewer.inspect( (DAMLCommon) lstAllInstances.getSelectedValue() );
    }

    void mnuItemCloseAll_actionPerformed(ActionEvent e) {
         m_viewer.closeInspectors();
    }

    void mnuItemOpenURL_actionPerformed(ActionEvent e) {
        String url = JOptionPane.showInputDialog( this, "Enter the URL:", "Jena DAML Browser", JOptionPane.PLAIN_MESSAGE );
        if (url != null  &&  url.length() > 0) {
            loadURL( url );
        }
    }

    void mnuItemProperties_actionPerformed(ActionEvent e) {
        JOptionPane.showMessageDialog( this, "Sorry, this feature not implemented yet.  Feel free ..." );
    }


    //==============================================================================
    // Inner class definitions
    //==============================================================================


    /**
     * The action of selecting an entry from the window menu is to show the menu
     */
    private class WindowMenuActionListener
        implements ActionListener
    {
        private DAMLInspector m_inspector = null;

        private WindowMenuActionListener( DAMLInspector inspector ) {
            m_inspector = inspector;
        }

        public void actionPerformed( ActionEvent e ) {
            m_inspector.reveal();
        }
    }




}
