/*
 *  (c) Copyright Hewlett-Packard Company 2001, 2002
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Created on 27 January 2001
 */

import com.hp.hpl.mesa.rdf.jena.common.SelectorImpl;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;

/** Tutorial 10 - demonstrate a container
 *
 * @author  bwm
 * @version Release='$Name:  $' Revision='$Revision: 1.1 $' Date='$Date: 2002/04/29 16:21:33 $'
 */
public class Tutorial10 extends Object {
    
    static final String inputFileName = "vc-db-1.rdf";
    
    public static void main (String args[]) {
       try {
            // create an empty model
            Model model = new ModelMem();
           
            // use the class loader to find the input file
            InputStream in = Tutorial10.class
                                       .getClassLoader()
                                       .getResourceAsStream(inputFileName);
            if (in == null) {
                throw new IllegalArgumentException(
                                       "File: " + inputFileName + " not found");
            }
            
            // read the RDF/XML file
            model.read(new InputStreamReader(in), "");
            
            // create a bag
            Bag smiths = model.createBag();
            
            // select all the resources with a VCARD.FN property
            // whose value ends with "Smith"
            StmtIterator iter = model.listStatements(
                new 
                    SelectorImpl(null, VCARD.FN, (RDFNode) null) {
                        public boolean selects(Statement s) {
                            try {
                                return s.getString()
                                        .endsWith("Smith");
                            } catch (RDFException e) {
                                throw new RDFError(e);
                            }
                        }
                    });
            // add the Smith's to the bag
            while (iter.hasNext()) {
                smiths.add(iter.next().getSubject());
            }
            
            // print the graph as RDF/XML
            model.write(new PrintWriter(System.out));
            System.out.println();
            
            // print out the members of the bag
            NodeIterator iter2 = smiths.iterator();
            if (iter2.hasNext()) {
                System.out.println("The bag contains:");
                while (iter2.hasNext()) {
                    System.out.println("  " +
                        ((Resource) iter2.next())
                                         .getProperty(VCARD.FN)
                                         .getString());
                }
            } else {
                System.out.println("The bag is empty");
            }
        } catch (Exception e) {
            System.out.println("Failed: " + e);
        }
    }
}