/*
 *  (c) Copyright Hewlett-Packard Company 2001 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
   *$Id: ARPFilter.java,v 1.4 2002/01/04 12:47:52 jjc Exp $
   
   AUTHOR:  Jeremy J. Carroll
*/
/*
 * ARPFilter.java
 *
 * Created on June 21, 2001, 10:01 PM
 */

package com.hp.hpl.jena.rdf.arp;



import java.util.Map;
import java.util.HashMap;
import java.util.BitSet;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import org.xml.sax.ErrorHandler;
import org.xml.sax.XMLReader;
//import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXNotRecognizedException;

import org.xml.sax.helpers.XMLFilterImpl;

import java.io.IOException;

/**
 *
 * @author  jjc
 */
class ARPFilter extends XMLFilterImpl 
                       implements RDFParserConstants, ARPErrorNumbers {
   static {
      org.apache.xerces.utils.XMLCharacterProperties.initCharFlags();
   }
  
    ARPFilter ()  
    {
        super(saxDriver());
        setErrorHandler(new DefaultErrorHandler());
    }

    
    static private XMLReader saxDriver()  {
        /*
        String parserClassName = System.getProperty("org.xml.sax.driver",
                                  "org.apache.xerces.parsers.SAXParser");    
        return XMLReaderFactory.createXMLReader(parserClassName);
         */
        return new org.apache.xerces.parsers.SAXParser();
    }
 
    
    private Throwable parsersLastException;
    
    private boolean embedding = true;
    boolean setEmbedding(boolean x) {
        boolean old = embedding;
        embedding = x;
        return old;
    }
    
    synchronized public void parse (InputSource input)
    throws IOException, SAXException
    {
        // Make sure we have a sane state for
        // Namespace processing.
        getParent().setFeature("http://xml.org/sax/features/namespaces", true);
        getParent().setFeature("http://xml.org/sax/features/namespace-prefixes", false);
        
        String base = input.getSystemId();
        if (base==null)
            warning(WARN_XMLBASE_MISSING,"Base URI not specified for input file; local references will expand incorrectly.");
        else 
            base = ParserSupport.truncateXMLBase(base);
            
        documentContext = new XMLContext(base,"");
        // Start the RDFParser
        pipe = new TokenPipe(this);
        parsersLastException = null;
        allCollections = new HashMap();
        allAboutEach = new Vector();
        final Thread xmlParser = Thread.currentThread();
        Thread rdfParser = new Thread(){
            public void run() {
                try {
                    RDFParser p = new RDFParser(pipe,ARPFilter.this);
                    if ( embedding )
                       p.content( documentContext );
                    else 
                        p.rdfFile( documentContext );
                }
                catch (WrappedException wrapped) {
                    xmlParser.interrupt();
                    parsersLastException = wrapped;
                }
                catch (ParseException parse) {
                    parsersLastException = parse;
                    xmlParser.interrupt();
                }
                catch ( CoRoutineDeathException death ) {
                }
                catch (Exception e) {
                    // Unnatural termiantion.
                    System.err.println("ARP - Internal Error: " + ParseException.formatMessage(e));
                    e.printStackTrace();
                    parsersLastException = e;
                    xmlParser.interrupt();
                    try {
                       getErrorHandler().fatalError(new SAXParseException("ARP internal error",null,e));
                    }
                    catch (SAXException saxe) {
                    }
                }
                catch (Throwable th) {
                    parsersLastException = th;
                    xmlParser.interrupt();
                }
            }
        };
        try {
            rdfParser.start();
            super.parse(input);
            pipe.putNextToken(new Token(EOF,null));
            rdfParser.join();
            // Wait for the RDFParser
        }
        catch ( InterruptedException e ) {
            // Might happen,
        }
        catch (CoRoutineDeathException e) {
            // Might happen,
        }
        catch ( WrappedException we ) {
            we.throwMe();
        }
        catch ( SAXException saxe ) {
            // From the XML parser
            warning( WARN_BAD_XML, "Failed to parse XML.");
            throw saxe;  // I think the handler will have seen this before.
        } 
        catch ( IOException ioe ) {
            throw ioe;
        }
        catch ( Throwable ee ) {
            System.err.println("ARP - Internal Error: " + ee.getMessage());
            ee.printStackTrace();
            parsersLastException = ee;
        }
        finally {
            // If the RDFParser is still going - interrupt it.
            rdfParser.interrupt();
            locator = null;
        }
        if ( parsersLastException != null ) {
            if ( parsersLastException instanceof WrappedException ) {
                ((WrappedException)parsersLastException).throwMe();
            }
            if ( parsersLastException instanceof RuntimeException )
                throw (RuntimeException)parsersLastException;
            if ( parsersLastException instanceof Error )
                throw (Error)parsersLastException;
            if ( parsersLastException instanceof Exception ) {
                if ( !( parsersLastException instanceof SAXException ) )
                    parsersLastException = new SAXException((Exception)parsersLastException);
                if ( parsersLastException instanceof SAXParseException )
                    getErrorHandler().fatalError((SAXParseException)parsersLastException);
                else
                    throw (SAXException)parsersLastException;
            }
            //  throw parsersLastException;
            return;
        }
        
        doAboutEach();
    //    System.out.println("Parse completed");
        
    }
    
    StatementHandler setStatementHandler(StatementHandler sh) {
        StatementHandler old = statementHandler;
        statementHandler = sh;
        return old;
    }
    StatementHandler statementHandler =
    new StatementHandler() {
        public void statement(AResource s,AResource p,AResource o){
        }
        public void statement(AResource s,AResource p,ALiteral o){
        }
    };
    // accessed in ARPQname.
    XMLContext documentContext;
    //String documentURI;
    private TokenPipe pipe;
    private Locator locator;
    static final String rdfns = "http://www.w3.org/1999/02/22-rdf-syntax-ns#".intern();
    static final String xmlns = "http://www.w3.org/XML/1998/namespace".intern();
    static final Map rdfnames = new HashMap();
    static {
        rdfnames.put("Description",new Integer(E_DESCRIPTION));
        rdfnames.put("RDF",new Integer(E_RDF));
        rdfnames.put("li",new Integer(E_LI));
    }
    static final Set
    knownRDFProperties = new HashSet();
    static final Set
    knownRDFTypes = new HashSet();
    static {
        knownRDFTypes.add("Bag");
        knownRDFTypes.add("Seq");
        knownRDFTypes.add("Alt");
        knownRDFTypes.add("Property");
        knownRDFProperties.add("type");
        knownRDFTypes.add("Statement");
        knownRDFProperties.add("subject");
        knownRDFProperties.add("predicate");
        knownRDFProperties.add("object");
        knownRDFProperties.add("value");
    }
    // The order of these must match their occurrence in grammar rules.
    static private String specialAtts[] = {
        "base",
        "lang",
        "space",
        "ID",
        "about",
        "aboutEach",
        "bagID",
        "resource",
        "parseType",
        "type"
    };
    static private String specialNameSpaces[] = {
        xmlns,
        xmlns,
        xmlns,
        rdfns,
        rdfns,
        rdfns,
        rdfns,
        rdfns,
        rdfns,
        rdfns
    };
    static private int A_XMLSPACE=-1;
    static private int specialAttValues[] = {
        A_XMLBASE,
        A_XMLLANG,
        A_XMLSPACE,
        A_ID,
        A_ABOUT,
        A_ABOUTEACH,
        A_BAGID,
        A_RESOURCE,
        A_PARSETYPE,
        A_TYPE,
    };
    
    private Map allCollections; 
    private Vector allAboutEach;
    void addMember( String collectionURI, AResource member ) {
        Object m;
        if ( member.isAnonymous() ) {
            m = member;
        } else {
            m = member.getURI();
        }
        Vector s = (Vector)allCollections.get(collectionURI);
        if (s==null) {
            s = new Vector();
            allCollections.put(collectionURI,s);
        }
        s.add(m);
    }
    void addAboutEach( ARPResource a ) {
        allAboutEach.add(a);
    }
    private void doAboutEach() {
        Iterator it = allAboutEach.iterator();
        while (it.hasNext()) {
            ARPResource aboutEach = (ARPResource)it.next();
            String uri = aboutEach.getURI();
            Vector members = (Vector)allCollections.get(uri);
            if ( members == null ) {
                warning(WARN_EMPTY_ABOUT_EACH,
                "There was an aboutEach declaration for the collection: '" + uri + "'." +
                ParseException.eol + "Either this collection was not defined, or it is empty.");
            } else {
                Iterator mem = members.iterator();
                while ( mem.hasNext() ) {
                    Object o = mem.next();
                    AResource member;
                    if ( o instanceof AResource ) 
                        member = (AResource)o;
                    else {
                      //  try {
                        member = new BadURIReference((String)o);
                     //   }
                     //   catch (MalformedURIException e) {
                     //       warning(WARN_MINOR_INTERNAL_ERROR,"aboutEach handling logic error.");
                     //       continue;
                     //   }
                    }
                    aboutEach.applyAboutEach(member);
                }
            }
        }
    }
    private void warning(int id,String s)  {
        try {
          getErrorHandler().warning(new ParseException(id,s));
        }
        catch (SAXException e) {
            throw new WrappedException(e);
        }
    }
    // a bit excessive in length!
    static private int defaultErrorMode[] = new int[400];
    static {
        for (int i=0; i<defaultErrorMode.length; i++)
            defaultErrorMode[i] = i/100;
    }
    private int errorMode[] = (int[])defaultErrorMode.clone();
    void setDefaultErrorMode() {
        errorMode = (int[])defaultErrorMode.clone();
    }
    void setLaxErrorMode() {
        setDefaultErrorMode();
        for (int i=100;i<200;i++)
            setErrorMode(i,EM_IGNORE);
        setErrorMode(WARN_MINOR_INTERNAL_ERROR,EM_WARNING);
    }
    void setStrictErrorMode() {
        setStrictErrorMode(EM_IGNORE);
    }
    
    void setStrictErrorMode(int nonErrorMode) {
        setDefaultErrorMode();
        for (int i=1;i<100;i++)
            setErrorMode(i,nonErrorMode);
        for (int i=100;i<200;i++)
            setErrorMode(i,EM_ERROR);
        int warning = EM_WARNING;
        int error = EM_ERROR;
        switch(nonErrorMode) {
            case EM_ERROR:
                warning = EM_ERROR;
                break;
            case EM_FATAL:
                warning = error = EM_FATAL;
                break;
        }
        setErrorMode(IGN_XMLBASE_USED,warning);
        setErrorMode(IGN_XMLBASE_SIGNIFICANT,error);
        setErrorMode(WARN_MINOR_INTERNAL_ERROR,warning);
        setErrorMode(WARN_DEPRECATED_XMLLANG,warning);
        setErrorMode(WARN_EMPTY_ABOUT_EACH,nonErrorMode);
        setErrorMode(WARN_UNKNOWN_PARSETYPE,warning);
        setErrorMode(WARN_BAD_XML,nonErrorMode);
        setErrorMode(WARN_LI_AS_TYPE,error);
    }
    int setErrorMode( int errno, int mode ) {
        int old = errorMode[errno];
        switch ( mode ) {
            case EM_WARNING:
            case EM_IGNORE:
                if ( errno >= 100 * EM_ERROR 
                  && errno != ERR_NOT_WHITESPACE ) 
                    break;
            case EM_ERROR:
            case EM_FATAL:
                switch ( errno ) {
                    case ERR_UNABLE_TO_RECOVER:
                        break;
                    default:
                   errorMode[errno] = mode;
                }
        }
        return old;
    }
    void parseWarning(int id,Location where,String s) throws ParseException {
        int mode = errorMode[id];
        if ( mode == EM_IGNORE )
            return;
        ParseException pe = new ParseException(id,where,s);
        if ( mode == EM_FATAL ) {
            pe.setFatal(true);
            mode = EM_ERROR;
        }
        if ( mode == EM_ERROR ) 
            throw pe;
        
        try {
            getErrorHandler().warning(pe);
            return;
        }
        catch (ParseException rethrown) {
            if ( rethrown == pe )
               throw rethrown;
            throw new WrappedException(pe);
        }
        catch (SAXException e) {
            throw new WrappedException(e);
        }
    }
    void parseWarning(Warn w) throws ParseException {
        parseWarning(w.number,w.location,w.msg);
    }
    private void putWarning(int no,Location where,String msg) {
        pipe.putNextToken(new Warn(no,where,msg));
    }
    public void setDocumentLocator (Locator locator)
    {
        this.locator = locator;
        super.setDocumentLocator(locator);
    }
    
    private void doSpecialAtt(int ix,int attName, String ns,BitSet attsDone, Attributes atts, Location where) 
     throws SAXException
    {
        
        attsDone.set(ix);
        
        if ( attName == A_XMLSPACE )
            return;
        
        pipe.putNextToken(new ARPQname(attName,where,ns,null,atts.getQName(ix)));
        String val = atts.getValue(ix);
        
        if ( attName == A_PARSETYPE ) {
            if ( val.equals("Resource") ) {
                pipe.putNextToken(new StrToken(AV_RESOURCE,where,val));
            } else if ( val.equals("daml:collection") ) {
                pipe.putNextToken(new StrToken(AV_DAMLCOLLECTION,where,val));
            } else {
                pipe.putNextToken(new StrToken(AV_LITERAL,where,val));
                if ( !val.equals("Literal") ) {
                    putWarning(WARN_UNKNOWN_PARSETYPE,where,"Unknown parseType: " + val);
                }
            }  
        } else  {
            pipe.putNextToken(new StrToken(AV_STRING,where,val));
        }
    }
    public void startElement (String uri, String localName, String rawName, Attributes atts)  throws SAXException {
        Location where = new Location(locator);
        putElementQname(uri,localName,rawName,where);
        BitSet attsDone = new BitSet();
        for (int i=0;i<specialAtts.length;i++) {
            int ix = atts.getIndex(specialNameSpaces[i],specialAtts[i]);
            if ( ix != -1) {
                doSpecialAtt(ix,specialAttValues[i],specialNameSpaces[i],attsDone,atts,where);
            }
            if ( specialNameSpaces[i]==rdfns ) {
                ix = atts.getIndex("",specialAtts[i]);
                if ( ix != -1) {
                    putWarning(WARN_UNQUALIFIED_RDF_ATTRIBUTE,where,"Unqualified use of rdf:"+atts.getLocalName(ix)+ " has been deprecated.");
                    doSpecialAtt(ix,specialAttValues[i],"",attsDone,atts,where);
                }
            }
        }
        for (int i=0; i<atts.getLength(); i++) {
            if ( !attsDone.get(i) ) {
                String ns = atts.getURI(i);
                String qn = atts.getQName(i);
                if ( qn.length()>=3 && qn.substring(0,3).toLowerCase().equals("xml") ) {
                    putWarning(WARN_UNKNOWN_XML_ATTRIBUTE,where,"XML attribute: " +atts.getQName(i)+" is not known and is being discarded.");
                    continue;
                }
                if ( ns.equals("") ) {
                    putWarning(WARN_UNQUALIFIED_ATTRIBUTE,where,"Attribute: " +atts.getLocalName(i)+". Unqualified use is deprecated. Assuming namespace: " + uri);
                    ns = uri;
                }
                putAttributeQname( ns, atts.getLocalName(i), qn,  where );
                pipe.putNextToken(
                new StrToken(AV_STRING,where,atts.getValue(i))
                );
            }
        }
    }
  
    public void endElement (String uri, String localName, String rawName)
    throws SAXException
    { Location loc = new Location(locator);
        pipe.putNextToken(new Token(E_END,loc));
    }
    public void characters (char ch[], int start, int length)
    throws SAXException
    { Location loc = new Location(locator);
            pipe.putNextToken(
            new StrToken( CD_STRING, loc, new String(ch,start,length) ) );
    }
    public void ignorableWhitespace (char ch[], int start, int length)
    throws SAXException
    {   // Never called.
        characters(ch,start,length);
    }
    private boolean isMemberProperty(String name) {
        if ( name.startsWith("_") ) {
            String number = name.substring(1);
            if (number.startsWith("-") || number.startsWith("0"))
                return false;
            try {
                Integer.parseInt(number);
                return true;
            }
            catch (NumberFormatException e) {
                try {
                    // It might be > Integer.MAX_VALUE
                    new java.math.BigInteger(number);
                    return true;
                }
                catch (NumberFormatException ee) {
                    return false;
                }
            }
        }
        return false;
    }
    private boolean isKnownRDFProperty(String name) {
        return knownRDFProperties.contains(name);
    }
    private void putElementQname(String uri, String localName, String q,Location where) 
    throws SAXException
    {  Token warn = null;
        if ( uri.equals(rdfns) ) {
            Integer val = (Integer)rdfnames.get(localName);
            if ( val == null ) {
                if ( isMemberProperty(localName) ) {
        pipe.putNextToken( new ARPQname(E_RDF_N,where,uri,localName, q) );
                   return;
                } else if ( !(knownRDFTypes.contains(localName)
                       ||isKnownRDFProperty(localName)) )
                    warn = new Warn(WARN_UNKNOWN_RDF_ELEMENT,where,"Creating statement(s) for unknown RDF element: '<rdf:"+localName+"'.");
            } else {
                pipe.putNextToken(new ARPQname(val.intValue(),where,uri,localName,q));
                return;
            }
        } 
        pipe.putNextToken( new ARPQname(E_OTHER,where,uri,localName, q) );
        if ( warn != null )
            pipe.putNextToken( warn );
    }
    
    private void putAttributeQname( String ns, String local, String q, Location where) 
    throws SAXException
    {
        if ( ns.equals(rdfns) ) {
            if ( isMemberProperty(local) ) {
        pipe.putNextToken( new ARPQname(A_RDF_N,where,ns,local,q) );
               return;
            } else if ( !isKnownRDFProperty(local) ) 
            putWarning(WARN_UNKNOWN_RDF_ATTRIBUTE,where,"Creating statement for unknown RDF property: 'rdf:"+local+"'.");
        }
        pipe.putNextToken( new ARPQname(A_OTHER,where,ns,local,q) );
    }
    
}

