/*
 *  (c) Copyright Hewlett-Packard Company 2001 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 * * $Id: ParserSupport.java,v 1.5 2002/01/04 12:47:53 jjc Exp $
   
   AUTHOR:  Jeremy J. Carroll
*/
/*
 * S.java
 *
 * Created on July 15, 2001, 7:13 AM
 */

package com.hp.hpl.jena.rdf.arp;

import java.util.Map;
import java.util.HashMap;
import org.apache.xerces.utils.XMLCharacterProperties;

import com.hp.hpl.jena.rdf.arp.lang.LanguageTag;
import com.hp.hpl.jena.rdf.arp.lang.LanguageTagSyntaxException;
import com.hp.hpl.jena.rdf.arp.lang.LanguageTagCodes;


/**
 *
 * @author  jjc
 * 
 */
class ParserSupport implements ARPErrorNumbers, RDFParserConstants, LanguageTagCodes {
   Map idsUsed = new HashMap();
    ParserSupport(ARPFilter arp) {
        this.arp = arp;
    }
    ARPFilter arp;
   void checkWhite(StrToken st) throws ParseException {
       String s = st.value;
       int lgth = s.length();
       int from = 0;
        // See XML specs for defn of whitespace.
        // Section 2.3
        // [3]    S    ::=    (#x20 | #x9 | #xD | #xA)+
        while ( from < lgth ) {
            switch (s.charAt(from++)) {
                case '\n':
                case '\r':
                case '\t':
                case ' ':
                    continue;
                    default:
                        throw new ParseException(ERR_NOT_WHITESPACE,st.location,
                          "Expected whitespace found: '" + s + "'."
                          );
            }
        }
   }

   void checkIdSymbol(XMLContext ctxt, StrToken s) throws ParseException {
      Location prev = (Location)idsUsed.get(s.value);
      if ( prev != null ) {
        arp.parseWarning(WARN_REDEFINITION_OF_ID,s.location,"Redefinition of ID: " + s.value);
        arp.parseWarning(WARN_REDEFINITION_OF_ID,prev,"Previous definition of '" + s.value + "'.");
      } else {
        idsUsed.put(s.value,s.location);
      }
      if ( !ctxt.isSameAsDocument() )
        arp.parseWarning(IGN_XMLBASE_SIGNIFICANT,s.location,"The use of xml:base changes the meaning of ID '" + s.value+ "'.");
        
      checkXMLName(s,s.value);
   }
   private void checkXMLName(StrToken s,String str) throws ParseException {
      if (!XMLCharacterProperties.validName(str))
         arp.parseWarning(WARN_BAD_NAME,s.location,"Not an XML Name: '"+str+"'");
   }
   void checkXMLLang(StrToken s) throws ParseException {
       String lang = s.value;
       try {
           LanguageTag tag = new LanguageTag(lang);
           int tagType = tag.tagType();
           if ( tagType == LT_ILLEGAL ) {
               arp.parseWarning(WARN_BAD_XMLLANG,s.location,tag.errorMessage());
           }
           if ( (tagType & LT_UNDETERMINED) == LT_UNDETERMINED ) {
                   arp.parseWarning(WARN_BAD_XMLLANG,s.location,"Unnecessary use of language tag \"und\" prohibited by RFC3066");
           }
           if ( (tagType & LT_IANA_DEPRECATED) == LT_IANA_DEPRECATED ) {
                   arp.parseWarning(WARN_DEPRECATED_XMLLANG,s.location,"Use of deprecated language tag \""+lang+"\".");
           }
           if ( (tagType & LT_PRIVATE_USE )  == LT_PRIVATE_USE ) {
                   arp.parseWarning(IGN_PRIVATE_XMLLANG,s.location,"Use of (IANA) private language tag \""+lang+"\".");
           } else if ( (tagType & LT_LOCAL_USE)  == LT_LOCAL_USE ) {
                   arp.parseWarning(IGN_PRIVATE_XMLLANG,s.location,"Use of (ISO639-2) local use language tag \""+lang+"\".");
           }else if ( (tagType & LT_EXTRA)  == LT_EXTRA ) {
                   arp.parseWarning(IGN_PRIVATE_XMLLANG,s.location,"Use of additional private subtags on language \""+lang+"\".");
           }
       }
       catch ( LanguageTagSyntaxException e ) {
           arp.parseWarning(WARN_MALFORMED_XMLLANG,s.location,e.getMessage());
       }
   }
   
   private String truncateXMLBase(StrToken s) {
      return truncateXMLBase(s.value);
   }
   static String truncateXMLBase(String rslt) {
      int hash = rslt.indexOf('#');
      if ( hash != -1 ) {
           return rslt.substring(0,hash);
      }
      return rslt;
   }
   XMLContext changeXMLBase(XMLContext ctxt, Token t)  
                         throws ParseException {
      arp.parseWarning(IGN_XMLBASE_USED,t.location,
           "Use of attribute xml:base is not envisaged in RDF Model&Syntax.");
      StrToken base = ((StrToken)t);
      String bb = URIref.encode(truncateXMLBase(base));
      try {
        ctxt = ctxt.withBase(bb);
      }
      catch (MalformedURIException mal) {
        arp.parseWarning(WARN_MALFORMED_URI,t.location,"Bad URI: " + mal.getMessage());
        ctxt = ctxt.revertToDocument();
      }
      return ctxt;
   }
   URIReference makeURIReference(XMLContext ctxt, Token t) 
                         throws ParseException { 
     StrToken s = (StrToken)t;
         String val = URIref.encode(s.value);
     try {
         URIReference rslt = new URIReference(ctxt,val);
         if ( val.indexOf(':') == -1 ) {
             if ( !ctxt.isSameAsDocument() ) {
                 URIReference other = new URIReference(ctxt.getDocument(),val);
                 if ( !other.equals(rslt) ) {
      arp.parseWarning(IGN_XMLBASE_SIGNIFICANT,t.location,
           "Use of attribute xml:base changes interpretation of relative URI: \"" + val + "\".");
                 }
             }
         }
         return rslt;
     }
     catch (MalformedURIException mal) {
        arp.parseWarning( WARN_MALFORMED_URI,
                          t.location,
                          "Bad URI: " + mal.getMessage()      );
        return new BadURIReference(val);
      }
   }
   
   void createTriple( ARPResource r, Token p, Object v, String reify )
   throws ParseException { 
       switch ( p.kind ) {
           case E_OTHER:
               r.setPredicateObject( ((ARPQname)p).asURIReference(arp), 
               v, 
               reify ); 
               break;
           case E_LI: 
               r.setLiObject( v, reify );
               break;
           case E_RDF_N:
               r.setMemberObject( 
               ((ARPQname)p).asURIReference(arp), 
               v, 
               reify ); 
               break;
               default:
                   throw new RuntimeException("Assertion failure in ParserSupport.createTriple");
       }
   }

   private Map checkNameSpace(StringBuffer b,ARPQname qn,Map ns) {
       String q = qn.qName;
       int colon = q.indexOf(':');
       String prefix = colon==-1?"":q.substring(0,colon);
       String old = (String)ns.get(prefix);
       if ( old == null || !old.equals(qn.nameSpace) ) {
           Map rslt = new HashMap(ns);
           rslt.put(prefix,qn.nameSpace);
           if ( prefix.length() == 0 ) {
               // MUST use \" as delimiter refer to RFC 2396, \' may appear in uri.
               b.append(" xmlns=\"" + qn.nameSpace + "\"");
           } else {
               // MUST use \" as delimiter refer to RFC 2396, \' may appear in uri.
               b.append(" xmlns:"+prefix+"=\"" + qn.nameSpace + "\"");
           }
           return rslt;
       } else {
           return ns;
       }
   }
   Map startLitEnt(StringBuffer b,Token t,Map ns) {
       ARPQname qn = (ARPQname)t;
       b.append("<"+qn.qName);
       return checkNameSpace(b,qn,ns);
   }
   void endLitEnt(StringBuffer b,Token t) {
       String q = ((ARPQname)t).qName;
       b.append("</"+q+">");
   }
   Map litAttrName(StringBuffer b,Token t,Map ns) {
       ARPQname qn = (ARPQname)t;
       Map rslt = checkNameSpace(b,qn,ns);
       b.append(" " + qn.qName );
       return ns;
   }
   void litAttrValue(StringBuffer b,Token t) {
       b.append("=\"" + encodeAttr(((StrToken)t).value) + "\"");
   }
   void litText(StringBuffer b,Token t) {
       b.append(encodeText(((StrToken)t).value));
   }
   Map xmlNameSpace() {
       Map rslt = new HashMap();
       rslt.put("xml",ARPFilter.xmlns);
       return rslt;
   }
/*
    private void checkXMLLiteralNameSpace(String uri,String raw) {
        if ( !uri.equals("") ) {
            int colon = raw.indexOf(':');
            String prefix = colon==-1?null:raw.substring(0,colon);
            String oldUri = (String)xmlLiteralNameSpaces.get(prefix);
            if (oldUri!=null && oldUri.equals(uri))
                return;
            thisDepthXMLLiteralNameSpaces.add(new String[]{prefix,oldUri});
            xmlLiteralNameSpaces.put(prefix,uri);
            if ( prefix == null ) {
                xmlLiteralValue.append(" xmlns");
            } else {
                xmlLiteralValue.append(" xmlns:");
                xmlLiteralValue.append(prefix);
            }
            xmlLiteralValue.append("='");
            xmlLiteralValue.append(encodeAttr(uri));
            xmlLiteralValue.append('\'');
        }
    }
*/
   // XML canonicalization attribute value processing rules.
   // http://www.w3.org/TR/2001/REC-xml-c14n-20010315#ProcessingModel
    static private String encodeAttr(String s) {    
        s = replace(s, "&", "&amp;");
        s = replace(s, "<", "&lt;");
       // s = replace(s, ">", "&gt;");
        s = replace(s, "\"", "&quot;");
        s = replace(s, "\t", "&#x9;");
        s = replace(s, "\n", "&#xA;");
        s = replace(s, "\r", "&#xD;");
        return s;
    }
    // XML canonicalization text node processing rules.
   // http://www.w3.org/TR/2001/REC-xml-c14n-20010315#ProcessingModel
    static private String encodeText(String s) {    
        s = replace(s, "&", "&amp;");
        s = replace(s, "<", "&lt;");
        s = replace(s, ">", "&gt;");
        s = replace(s, "\r", "&#xD;");
        return s;
    }   
    // copied from Brian's code.
    private static String replace(String s, String oldString, String newString) {
        String result = "";
        int length = oldString.length();
        int pos = s.indexOf(oldString);
        int lastPos = 0;
        while (pos >= 0) {
            result = result + s.substring(lastPos, pos) + newString;
            lastPos = pos + length;
            pos = s.indexOf(oldString, lastPos);
        }
        return result + s.substring(lastPos, s.length());
    }


}
