/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * BaseTest.java
 *
 * Created on November 28, 2001, 10:15 AM
 */

package com.hp.hpl.jena.rdf.arp.test;

import junit.framework.*;

import java.io.*;

import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.mem.*;
import com.hp.hpl.jena.rdf.arp.*;

import org.xml.sax.*;

/**
 * This is the base class for various tests.
 * It knows how to:
 *  - read an RDF/XML file
 *  - creating a JenaModel
 *  - asserting a JUnit failure if there is an error/warning
 *
 * @author  jjc
 * 
 */
abstract class BaseTest extends TestCase implements RDFErrorHandler {

    private String xmlBase;
    private InputStream in;
    Model model;
    public BaseTest(String xmlBase,InputStream file) {
        super(xmlBase);
        in = file;
        this.xmlBase = xmlBase;
    }
    abstract protected void runTest();
    
    void loadRDF() throws IOException, RDFException {
        model = new ModelMem();
        JenaReader jr = new JenaReader();
        jr.setErrorHandler(this);
        jr.setProperty("error-mode","strict");
        jr.read(model,in,xmlBase);
        in.close();
    }
    
    
    // RDFErrorHandler implementation
    
    public void warning(Exception e) {
        error(0,e);
    }
    public void error(Exception e) {
        error(1,e);
    }
    public void fatalError(Exception e) {
        error(2,e);
    }
    private void error(int level,Exception e) {
        if ( e instanceof ParseException ) {
            onError(((ParseException)e).getErrorNumber());
        } else if ( e instanceof SAXParseException ) {
            onError(ARPErrorNumbers.WARN_BAD_XML);
        } else if ( e instanceof SAXException ) {
            fail("Not expecting a SAXException: " + e.getMessage());
        } else {
            fail("Not expecting an Exception: " + e.getMessage());
        }
    }
    
    void onError(int num) {
        fail("Parser reports unexpected error: "+num);
    }
}
