/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*
 * BugzillaTestSuite.java
 *
 * Created on November 28, 2001, 10:00 AM
 */

package com.hp.hpl.jena.rdf.arp.test;

import junit.framework.*;
import com.hp.hpl.jena.rdf.arp.ARPErrorNumbers;
/**
 *
 * @author  jjc
 */
public class BugzillaTestSuite extends TestSuite implements ARPErrorNumbers{
    static private String dir = "modules/rdf/regression/arp/";
    /** Creates new BugzillaTestSuite */
    public BugzillaTestSuite() {
        super("ARP reported bugs");
        addBad("Fragment Namespace URI",500,WARN_RELATIVE_NAMESPACE_URI_DEPRECATED);
        addBad("rdf:_-1",670,WARN_UNKNOWN_RDF_ATTRIBUTE);
        addBad("rdf:_0",675,WARN_UNKNOWN_RDF_ATTRIBUTE);
        addBad("rdf:_01",676,WARN_UNKNOWN_RDF_ATTRIBUTE);
        addBad("Bad rdf:_NNN",677,0).setExpected(
           new int[]{
               WARN_UNKNOWN_RDF_ELEMENT,
               WARN_UNKNOWN_RDF_ELEMENT,
               WARN_UNKNOWN_RDF_ELEMENT } );
        addBad("attr rdf:-1",671,WARN_BAD_XML);
        addBad("attr rdf:0",672,WARN_BAD_XML);
        addBad("element rdf:-1",673,WARN_BAD_XML);
        addBad("element rdf:0",674,WARN_BAD_XML);
        addBad("rdf:parseType=\"literal\"",680,WARN_UNKNOWN_PARSETYPE);
        addBad("rdf:li as type",690,WARN_LI_AS_TYPE);
        addTriple("colon in ID",740);
        addTriple("xml:base with no path",510);
        addEqual("unicode and other encodings",new int[]{730,731,732});
    }
    
    static public TestSuite suite() {
        return new BugzillaTestSuite();
    }
    
    private ErrorTest addBad(String name,int id, int err) {
        int bugNo = id/10;
        int frag = id%10;
        String fName = "bad-bug"+bugNo+"_"+frag+".rdf";
        ErrorTest et = new ErrorTest("http://example.org/"+fName,new LazyFileInputStream(dir+fName),err);
        et.setName(name);
        if (err==WARN_BAD_XML) {
            et.setExpected(new int[]{WARN_BAD_XML,WARN_BAD_XML});
       }
        addTest(et);
        return et;
    }
    
    private void addTriple(String name,int id) {
        int bugNo = id/10;
        int frag = id%10;
        String fName = "bug"+bugNo+"_"+frag;
        NTripleTest nt = new NTripleTest(id==510?"http://example.org":("http://example.org/"+fName+".rdf"),
                             dir+fName+".rdf",
                             dir+fName+".nt");
        nt.setName(name);
        addTest(nt);
    }
    
    private void addEqual(String name,int ids[]) {
        String files[] = new String[ids.length];
        for (int i=0;i<ids.length;i++)
            files[i] = "eq-bug"+ids[i]/10+"_"+ids[i]%10+".rdf";
        addTest(new TestEqualitySuite(name,dir,"http://example.org/",files));
    }

}
