/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * ErrorTest.java
 *
 * Created on November 28, 2001, 9:58 AM
 */

package com.hp.hpl.jena.rdf.arp.test;

import java.io.InputStream;

import com.hp.hpl.mesa.rdf.jena.model.*;
import java.io.*;

/**
 *
 * @author  jjc
 */
class ErrorTest extends BaseTest {

    private int expected[];
    private int errorCnt = 0;
    public ErrorTest(String xmlBase,InputStream e) {
        super(xmlBase,e);
    }
    public ErrorTest(String xmlBase,InputStream e,int expected) {
        super(xmlBase,e);
        setExpected(new int[]{expected});
    }
    
    void setExpected(int i) {
        setExpected(new int[]{i});
    }
    void setExpected(int ex[]) {
        expected = ex;
    }
    protected void runTest() {
        try {
            loadRDF();
        }
        catch (RDFException re) {
         //   System.out.println(re.toString());
            if ( re.getErrorCode()==re.NESTEDEXCEPTION
            && re.toString().indexOf("SAXException")!=-1 ) {
                // ignore.
            } else {
                
                fail(re.getMessage());
            }
        }
        catch (IOException ioe) {
            fail(ioe.getMessage());
        }
        if (expected != null)
            for (int i=0;i<expected.length;i++) {
                if (expected[i]!= -1)
                    fail("Expected error number " + expected[i] + ", was not detected.");
            }
        else if (errorCnt == 0 )
            fail("Did not detect errors in bad input file");
    }
    void onError(int id) {
        errorCnt++;
        if (expected != null) {
            for (int i=0;i<expected.length;i++) {
                if (expected[i]==id) {
                    expected[i] = -1;
                    return;
                }
            }
            super.onError(id);
        }
    }
    
    
    
    

}
