/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * WGTestSuite.java
 *
 * Created on November 28, 2001, 10:00 AM
 */

package com.hp.hpl.jena.rdf.arp.test;

import junit.framework.*;
import java.util.zip.*;
import java.io.*;

/**
 *
 * @author  jjc
 */
public class WGTestSuite  extends TestSuite{

    /** Creates new WGTestSuite 
        This is a private snapshot of the RDF Test Cases Working Draft's
        data.
     *Note the special hack that blocks rdf-ns-prefix-confusion/test0008.rdf,
     *it is very ugly. I assume that this code will be dead by March 2002,
     *if not the hack should be redone more pleasantly.
     */
    private WGTestSuite() {
        super("WG Parser Tests");
        try {
           zip = new ZipFile("modules/rdf/regression/arp/latest_Approved.zip");
        }
        catch (IOException ioe) {
          //  System.err.println("NN");
            zip = null; // will get a null pointer error later.
        }
        add("amp-in-url",1,0);
        add(3,"rdf-containers-syntax-vs-schema",8,1,5);
        add("rdfms-empty-property-elements",15,3);
        add("rdfms-identity-anon-resources",5,0);
        add(4,"rdf-ns-prefix-confusion",14,9,8);
        add("rdfs-domain-and-range",2,0);
        add("rdfs-no-cycles-in-subClassOf",1,0);
        add("rdfs-no-cycles-in-subPropertyOf",1,0);
    }
    private ZipFile zip;
    static public TestSuite suite() {
        return new WGTestSuite();
    }
    private void add(String dir,int nt,int errs) {
        add(3,dir,nt,errs,0);
    }
    
    /**
     * @param omit Either 0 or the number of an nt case that is deprecated.
     * @param pad  The field width of the 0 padded number.
     */
    private void add(int pad,String dir,int nt,int errs,int omit) {
        addTest(new WGTestCaseDir(
        dir+"/",
        zip,
        "http://www.w3.org/2000/10/rdf-tests/rdfcore/"+dir+"/",
        array(pad,"test",nt,omit),
        array(pad,"error",errs,0)));
    }
    
    private String[] array(int pad,String base,int num,int omit) {
        String rslt[] = new String[num-(omit!=0?1:0)];
        for (int i=1;i<=num;i++)
            rslt[i-((omit!=0&&i>omit)?2:1)] = base + pad(pad,i);
        return rslt;
    }
    private String pad(int p,int val) {
       
        int vv = val;
        String rslt = "";
        while ( vv > 0 ) {
            p--;
            vv /= 10;
        }
        for (int i=0;i<p;i++)
            rslt += "0";
        rslt += val;
        return rslt;
    }
}
