//--HEADER--
/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * $Id: TestQuery.java,v 1.1 2001/09/19 09:28:33 afs Exp $
 */
//--HEADER--

package com.hp.hpl.jena.rdf.query.test;

// Unsubtle JUnit-ization of some test code.
// This will be replaced by a more general testing of queries.

import java.io.* ;
import junit.framework.* ;

import com.hp.hpl.jena.rdf.query.* ;
import com.hp.hpl.jena.rdf.query.parser.* ;

import java.io.* ;
import java.util.* ;

import com.hp.hpl.mesa.rdf.jena.model.* ;
import com.hp.hpl.mesa.rdf.jena.common.* ;
import com.hp.hpl.mesa.rdf.jena.mem.* ;


public class TestQuery extends TestCase
{
    Model model = null ;
    public boolean dumpModel = false ;
    public boolean verbose = false ;

    public TestQuery(String name)
    {
        super(name) ;
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite() ;
        suite.addTest(new TestQuery("testQuery"));
        return suite ;
    }

    public static void addTests(TestSuite suite)
    {
        suite.addTest(new TestQuery("RDQL Query "));
    }

    protected void runTest() throws Throwable
    {
        List queries = new ArrayList() ;
        queries.add("SELECT ?x, ?z WHERE (?x, ?a, ?b), (?b, ?y, ?z)") ;
        queries.add("SELECT ?z WHERE (?x, ?y, ?z) AND ?z == 1") ;
        queries.add("SELECT ?z WHERE (<http://never/r-1>, <http://never/p-0>, ?z)") ;

        // Contents of a bag
        queries.add("SELECT ?x, ?y WHERE (<http://never/bag>, ?x, ?y) AND ?y ne <http://www.w3.org/1999/02/22-rdf-syntax-ns#Bag>") ;
        // Same
        queries.add("SELECT ?x, ?y WHERE (<http://never/bag>, ?x, ?y) AND ?x ne <http://www.w3.org/1999/02/22-rdf-syntax-ns#type>") ;

        for ( Iterator qIter = queries.iterator() ; qIter.hasNext() ; )
        {
            String qs = (String)qIter.next() ;
            if ( verbose )
            {
                System.out.println() ;
                System.out.println("Query:") ;
                System.out.println(qs) ;
            }
            // Currently "success" is executing the query at all!
            Query query = new Query(qs) ;
            query.setSource(model);
            QueryEngine qe = new QueryEngine(query) ;
            QueryResults results = qe.exec() ;
            QueryResultsFormatter fmt = new QueryResultsFormatter(results) ;
            fmt.consume();
        }
    }

    public void setUp() throws Exception
    {
        //System.out.println() ;
        //System.out.println("TestQuery") ;
        model = makeModel() ;
    }

    static public Model makeModel() throws Exception
    {
        Model model = new ModelMem() ;


        // Resource loop
        for ( int i = 0 ; i < 2 ; i++ )
        {
            // Property loop
            for ( int j = 0 ; j < 2 ; j++ )
            {
                model.add(model.createResource("http://never/r-"+i) ,
                          model.createProperty("http://never/p-"+j) ,
                          //"val-r-"+i+"-p-"+j) ;
                          i+j) ;
            }
        }

        // Bag statements are added to the model.
        Bag bag = model.createBag("http://never/bag") ;
        bag.add("11") ;
        bag.add("22") ;

        // Path.
        model.add(model.createResource("http://never/path"),
                  model.createProperty("http://never/path") ,
                  model.createResource("http://never/r-0")) ;
        return model ;
    }

    public void tearDown()
    {
        model = null ;
    }
}
