/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: TestWriterXMLBaseProp.java,v 1.1 2002/01/09 16:55:33 bwm Exp $
 */

package com.hp.hpl.mesa.rdf.jena.common.regression;

import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.common.regression.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

/** 
 * @author bwm
 * @version $Name:  $ $Revision: 1.1 $ $Date: 2002/01/09 16:55:33 $
 */
public class TestWriterXMLBaseProp extends TestCase {
    
    protected Model model = null;
    String base1 = "http://example/foobar";
    String base2 = "http://example/barfoo";
    
    public TestWriterXMLBaseProp(String name) {
        super(name);
    }
    
    public static Test suite() {
        return new TestSuite(TestWriterXMLBaseProp.class); 
    }
    
    public void testRDFXMLWriter() {
        doTest("RDF/XML");
    }
    
    public void doTest(String lang) {
        File file = null;
        try {
            model = new ModelMem();
            RDFWriter writer = model.getWriter(lang);
            String oldvalue = (String) writer.setProperty("xmlbase", base1);
            assertTrue(oldvalue == null);
            
            oldvalue = (String) writer.setProperty("xmlbase", base2);
            assertEquals(base1, oldvalue);
        
            file = File.createTempFile("rdf", ".xml");
            file.deleteOnExit();
        
            FileWriter fwriter = new FileWriter(file);
            writer.write(model, fwriter , "");
            fwriter.close();
        
            FileReader freader = new FileReader(file);
        
            String rdf = "";
            int i = freader.read();
            while (i>=0) {
                rdf = rdf + (char) i;
                i = freader.read();
            }
            freader.close();
            // this code makes some assumptions about the formating
            // which it shouldn't, but I haven't time to write the
            // sax code to do it properly
            if ( rdf.indexOf("xml:base=\"" + base2 + "\"") < 0 &&
                 rdf.indexOf("xml:base='"  + base2 + "'") < 0) {
                assertTrue(false);
            }
        } catch (Exception e) {
            e.printStackTrace(System.out);
            assertTrue(false);
        } finally {
           file.delete();
        }
    }
}